#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import os
import glob
os.environ['PYTHON_EGG_CACHE'] = '/tmp/.python-eggs'
sys.dont_write_bytecode = True
sys.path += glob.glob(os.path.join(os.path.dirname(__file__),'*/*.egg'))

import gevent
import gevent.event
import gevent.coros
import gevent.queue
import gevent.monkey
gevent.monkey.patch_all(dns=False)

import re
import time
import zlib
import gc
import io
import logging
import logging.handlers
import signal
import thread
import threading
import Queue
import json
import collections
import optparse
import socket
import ConfigParser
import traceback
import base64
import fnmatch
try:
    import psutil
except:
    psutil = None

import S3
import SQS
from lru_cache import lru_cache
from agent_config import AgentConfig
from file_operate import FailedTimestampFile
from file_operate import Configer
from iwss_log_converter import IwssLogConverter
from ics_log_converter import ICSLogConverter
from deepedge_log_converter import DeepEdgeLogConverter
from iwss_send_event_log import IwssSendEventLog
from enums import *
import uuid
import urllib2
from de_log import DEViolationLog,DEAccessLog,DEAppbandLog
from logFiltering.logFilteringByHits import LogFilteringByHits
from logFiltering.logFilteringByAttributes import LogFilteringByAttributes
from logFiltering.logFilteringManager import getLogFilteringManager


MAX_LOG_FLUSHTIME = 70
MAX_LOG_FAILED_QUEUE_SIZE = 200
MAX_UPLOAD_WAIT = 5
MAX_UPLOAD_RETRY = 3
MAX_UPLOAD_FAILWAIT = 10
ICS_LOG_FILENAME_PREFIX = '/etc/iscan/log/report.access'
CONFIGFILE="/etc/iscan/intscan.ini"
intscanConfig=ConfigParser.ConfigParser()
intscanConfig.read(CONFIGFILE)
#IWSS_LOG_FILENAME_PREFIX = '/etc/iscan/log/report_log'
IWSS_LOG_FILENAME_PREFIX = '/etc/iscan/log/' + intscanConfig.get('Common','report_log')
logging.info('FILENAME_PREFIX: %s', IWSS_LOG_FILENAME_PREFIX)
IWSS_LOG_FILENAME_PREFIX_MIGRATION = '/var/iwss/log_migration/report_log'
BUCKET_KEYNAME_PREFIX = 'scanner'
IWSSD_PROCESS_NAME = 'iwssd'
CASSA_KEYSPACE = 'Profiles'
CASSA_COLUMN_FAMILY = 'companies'
CASSA_COLUMN_HOMEDC = 'homeDc'

PRODUCT = Product.IWSS
STORAGE_TYPE = DeployMode.ALL_IN_ONE
LOG_SERVER = '127.0.0.1:6011'
LOG_REDUCTION = 1

manager = getLogFilteringManager()
filteringByHits = LogFilteringByHits()
#filteringByHits = LogFilteringByHits()
filteringByHits.setLogFilteringConfiguration({
     '0': {'enable': True} # 'cid'
})
filteringByAttrs = LogFilteringByAttributes()
filteringByAttrs.setLogFilteringConfiguration({
     '0': { # 'cid'
         'enable': True,
         'setting': {
             'CLF_ContentType': []
         }
     }
})

modules = [filteringByHits, filteringByAttrs]
manager.registerLogFiltering(modules)

# ics      : (timestamp, bucket, queue, sqshost) -> lines dict, type {int:[str]}
# iwss     : (timestamp) -> lines dict, type {int:[str]}
# deep edge: pass
LOGBLOCK_LINES  = collections.defaultdict(list)
LOGBLOCK_TIMER  = collections.defaultdict()
# ics      : (timestamp, bucket, queue, sqshost) -> uploader coroutine, type greenlet
# iwss     : (timestamp) -> uploader
# deep edge: pass
LOGBLOCK_GREENLETS  = {}
# ics      : [(timestamp, bucket, queue, sqshost, ip, zdata)]
# iwss     : [(timestamp, zdata)]
# deep edge: pass
LOGBLOCK_FAILED = collections.deque(maxlen=MAX_LOG_FAILED_QUEUE_SIZE)


MSGTYPE_LIST=[[65537],[65538,524289,720897,327681],[131073],[196609,196610,196616,196624,589825,196612,458753,720898,262145,196618,851969],[393217],[655361,786433,786434]]

def _getip(ifname='eth0'):
    try:
        import fcntl, socket, struct
        SIOCGIFADDR = 0x8915
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        ip = socket.inet_ntoa(fcntl.ioctl(sock.fileno(), SIOCGIFADDR, struct.pack('256s', ifname[:15]))[20:24])
        sock.close()
        return ip
    except (ImportError, IOError):
        import socket
        return socket.gethostbyname(socket.gethostname())

def _getpids(process_name=IWSSD_PROCESS_NAME):
    """return pid list of iwssd
    'iwssd' -> ['1234', '1235', '1236']
    """
    assert isinstance(process_name, basestring) and re.match(r'\w+', process_name)
    if psutil:
        pids = [x.pid for x in psutil.process_iter() if x.name==process_name]
    else:
        pids = os.popen('pgrep %s' % process_name).read().splitlines()
    return [str(x) for x in pids]

def _getfilename(taskname, product):
    if product == Product.ICS:
        """For ICS: return today filename of pid
        '1001' -> '/etc/iscan/log/report.access.1001.20120326'
        '1984' -> '/etc/iscan/log/report.access.1984.20120326'
        """
        pid = taskname
        assert isinstance(pid, basestring) and re.match(r'\d+', pid)
        today = time.strftime('%Y%m%d', time.localtime())
        return '%s.%s.%s' % (ICS_LOG_FILENAME_PREFIX, pid, today)
    elif product == Product.IWSS:
        """For IWSS: return today filename of logname
        IWSS_APPLICATION_LOG -> '/etc/iscan/log/report_log.app.20120326'
        IWSS_ACCESS_LOG      -> '/etc/iscan/log/report_log.access.20120326'
        IWSS_VIOLATION_LOG   -> '/etc/iscan/log/report_log.violation.20120326'
        """
        logname = taskname
        assert isinstance(logname, basestring)
        today = time.strftime('%Y%m%d', time.localtime())
        return '%s.%s.%s' % (IWSS_LOG_FILENAME_PREFIX, logname, today)
    elif product == Product.DEEP_EDGE:
        pass
    else:
        logging.error('_getfilename error: taskname=%s, product=%s', taskname, product)

def _getnextfilename(filename):
    """return filename of nextday
    /etc/iscan/log/report.access.1.20120326  -> /etc/iscan/log/report.access.1.20120327
    /etc/iscan/log/report.access.2.20120331  -> /etc/iscan/log/report.access.2.20120401
    /etc/iscan/log/report.app.20120326       -> /etc/iscan/log/report.app.20120327
    /etc/iscan/log/report.access.20120326    -> /etc/iscan/log/report.access.20120327
    /etc/iscan/log/report.violation.20120326 -> /etc/iscan/log/report.violation.20120327
    """
    assert isinstance(filename, basestring)
    dirname, basename = os.path.split(filename)
    m = re.search(r'^(.+\.)(20\d{6})$', filename)
    assert m is not None
    today = m.group(2)
    nextday = time.strftime('%Y%m%d', time.localtime(time.mktime(time.strptime(today, '%Y%m%d'))+86400))
    nextday_filename = m.group(1) + nextday
    return nextday_filename

def _ics_doupload(s3_conn, bucket, sqs_conn, queue, ip, compress, timestamp, data):
    """ics_upload function, put (timestamp, data) to S3, and send message to SQS"""
    if not data:
        logging.error('_ics_doupload got an empty data=%r timestamp=%r', data, timestamp)
        return False
    logging.info('ics_uploader read a block, timestamp=%r, raw data size=%r', timestamp, len(data))
    if compress:
        data = zlib.compress(data, int(compress))
    key = '%s/%s_%s.log' % (BUCKET_KEYNAME_PREFIX, timestamp, ip)
    for retry in xrange(MAX_UPLOAD_RETRY):
        try:
            logging.info('S3 bucket put key=%r data size=%r, retry=%r', key, len(data), retry)
            response = s3_conn.put(bucket, key, S3.S3Object(data))
            if response.message == '200 OK':
                logging.info('S3 bucket put response.message=%r', response.message)
            else:
                logging.error('S3 bucket put failed: response.message=%r', response.message)
                continue
            # send SQS message to logwriter server
            data = {'timestamp':timestamp, 'ip':ip, 'bucket':bucket, 'key':key}
            logging.info('SQS queue write message(data=%s)', data)
            response = sqs_conn.send_message(queue, json.dumps(data))
            logging.info('SQS queue write message end, response.message=%r', response.message)
            return True
        except socket.error as e:
            logging.error('_ics_doupload log block error: %s', e)
            time.sleep(MAX_UPLOAD_FAILWAIT)
        except Exception as e:
            logging.exception(u'_ics_doupload log block error: %s', e)
            time.sleep(MAX_UPLOAD_FAILWAIT)
    else:
        LOGBLOCK_FAILED.append((timestamp, bucket, queue, sqs_conn.server, ip, data))

class common_upload_ops(object):
    def __init__(self):
        pass
    def upload_fail(self):
        pass
    def upload_recover(self):
        pass
    def upload_success(self):
        pass

class iwss_upload_ops(common_upload_ops):
    def __init__(self):
        super(iwss_upload_ops, self).__init__()
    def upload_fail(self):
        IwssSendEventLog.send(IwssSendEventLog.EVT_LOG_UPLOAD_FAIL)
    def upload_recover(self):
        IwssSendEventLog.send(IwssSendEventLog.EVT_LOG_UPLOAD_RECOVER)

class de_upload_ops(common_upload_ops):
    def __init__(self):
        super(de_upload_ops, self).__init__()

def _common_doupload(compress, timestamp, data, upload_ops):
    """
    common_upload function, put (timestamp, data) to local storage or log server.
    iwss and deep_edge are using this function
    """
    #wesson add for sync certificae error flie to postgresql
    logging.debug('enter _common_doupload')

    clientcert_filename = os.path.join("/usr/iwss/sql/postgresql/", "client_certifcate_error")
    dbsyncfilepath = os.path.join("/usr/iwss/sql/postgresql/", "syncclientcert.sh")
    if os.path.exists(clientcert_filename):
        if os.path.exists(dbsyncfilepath):
            try:
                logging.debug('will do sync to pgsql')
                ret = os.system(dbsyncfilepath);
                if ret == 0:
                    logging.debug('successfully do sync file to pgsql, delete file')
                    try:
                        os.remove(clientcert_filename)
                    except Exception as ee:
                        logging.debug('remove file error: %s', ee)
                else:
                    logging.error('error do sync file to pgsql, ret:%d',ret)
            except Exception as e:
                logging.exception('sync to pqsql error: %s', e)
        else:
            logging.error('dbsyncfilepath:%s do not exsit',dbsyncfilepath)
    else:
        logging.info('clientcert_filename:%s do not exsit',clientcert_filename)

    if not data:
        logging.error('_common_doupload got an empty data=%r timestamp=%r', data, timestamp)
        return False
    logging.debug('common_uploader read a block, timestamp=%r, raw data size=%r', timestamp, len(data))

    filename = ''
    postdata = {}
    if STORAGE_TYPE == DeployMode.ALL_IN_ONE:
        filename = os.path.join(AgentConfig.UPLOAD_DATA_DIR, ('%d_%s' % (timestamp, str(uuid.uuid1()))) )
        with open(filename, 'wb') as fp:
            fp.write(data)
            fp.close()
        postdata = {'ts' :timestamp, 'location':filename}
    elif STORAGE_TYPE == DeployMode.STAND_ALONE:
        if compress:
            compressed_data = base64.encodestring(zlib.compress(data, int(compress)))
            postdata = {'ts':timestamp, 'zip':1, 'content':compressed_data}
        else:
            postdata = {'ts':timestamp, 'zip':0, 'content':data}

    postdata = json.dumps(postdata)
    auth_headers = AgentConfig.getAuthHeader()
    logging.info('auth_header:%r',auth_headers)
    for retry in xrange(MAX_UPLOAD_RETRY):
        try:
            logUploadURL = ''.join(( AgentConfig.PROTOCOL, '://', LOG_SERVER, '/log_upload'))
            request = urllib2.Request(logUploadURL, postdata, auth_headers)
            fd = urllib2.urlopen(request, timeout = 1800)
            responseCode = fd.getcode()
            if responseCode == 200:
                if not AgentConfig.UPLOAD_STATUS:
                    upload_ops.upload_recover()
                    AgentConfig.UPLOAD_STATUS = True
                    Configer.set(AgentConfig.SYSEVENT_CONFIGFILE, 'UPLOAD_STATUS', 'status', AgentConfig.UPLOAD_STATUS)
                upload_ops.upload_success()
                logging.info("common_upload successfully: response.code=%d", responseCode)
                return True
            else:
                logging.info("common_upload failed: response.code=%d", responseCode)
        except socket.error as e:
            logging.error('_common_doupload log block error: %s', e)
        except urllib2.URLError as e:
            logging.error('_common_doupload log block error: %s, timestamp=%r', e, timestamp)
        except Exception as e:
            logging.exception(u'_common_doupload log block error: %s', e)
        time.sleep(MAX_UPLOAD_FAILWAIT)
    else:
        if AgentConfig.UPLOAD_STATUS:
            upload_ops.upload_fail()
            AgentConfig.UPLOAD_STATUS = False
            Configer.set(AgentConfig.SYSEVENT_CONFIGFILE, 'UPLOAD_STATUS', 'status', AgentConfig.UPLOAD_STATUS)
        if os.path.isfile(filename):
            os.remove(filename)
        FailedTimestampFile.write(timestamp, data)

def _common_doupload_skynet(compress, timestamp, data):
    # skynet stats calcuation is separated, so agent should upload l10n log to this process.
    logUploadURL = 'http://127.0.0.1:6013/upload/rawlog'
    postdata = {}
    if compress:
        compressedData = base64.encodestring(zlib.compress(data, int(compress)))
        postdata = {'ts':timestamp, 'zip':1, 'content':compressedData}
    else:
        postdata = {'ts':timestamp, 'zip':0, 'content':data}
    for retry in xrange(MAX_UPLOAD_RETRY):
        try:
            request = urllib2.Request(logUploadURL, json.dumps(postdata))
            fd = urllib2.urlopen(request, timeout=1800)
            responseCode = fd.getcode()
            if responseCode == 200:
                logging.debug('_common_doupload_skynet upload successfully: timestamp=%s', timestamp)
                return True
        except Exception as e:
            logging.error(traceback.format_exc())
            time.sleep(MAX_UPLOAD_FAILWAIT)
    else:
        logging.error('_common_doupload_skynet upload error: timestamp=%s', timestamp)
    return False

@lru_cache(maxsize=100)
def _query_homedc(cid, cassa):
    assert isinstance(cid, basestring)
    try:
        logging.info('_query_homedc try to connect cassa=%r', cassa)
        pool = pycassa.ConnectionPool(CASSA_KEYSPACE, server_list=cassa.split(','))
        cf   = pycassa.ColumnFamily(pool, CASSA_COLUMN_FAMILY)
        logging.info('_query_homedc(cid=%r, cassa=%r)', cid, cassa)
        homedc = cf.get(cid, [CASSA_COLUMN_HOMEDC])[CASSA_COLUMN_HOMEDC]
        homedc = json.loads(homedc)
        logging.info('_query_homedc(cid=%r, cassa=%r) return %s', cid, cassa, homedc)
        assert isinstance(homedc, dict)
        return homedc
    except Exception, e:
        logging.error('_query_homedc(cid=%r, cassa=%r) failed:%s', cid, cassa, e)
        return {}

def _ics_upload_by_timestamp(timestamp, bucket, queue, sqshost, awsid, awskey, s3_conn, sqs_conn_dict, sqs_conn_dict_lock, ip, compress):
    now = int(time.time())
    logging.info('now=%r is large enough, begin uploader for timestamp=%s', now, timestamp)
    data = ''.join(LOGBLOCK_LINES.pop((timestamp, bucket, queue, sqshost), []))
    LOGBLOCK_GREENLETS.pop((timestamp, bucket, queue, sqshost), [])
    if not data:
        logging.warning('LOGBLOCK_LINES.pop((timestamp=%r), []) return NULL!', timestamp)
        return
    if (sqshost, queue) not in sqs_conn_dict:
        with sqs_conn_dict_lock:
            if (sqshost, queue) not in sqs_conn_dict:
                logging.info('sqs_conn.connect(queue=%r) sqshost=%r', queue, sqshost)
                sqs_conn = SQS.AWSAuthConnection(awsid, awskey, False, server=sqshost)
                sqs_conn.connect(queue)
                sqs_conn_dict[(sqshost, queue)] = sqs_conn
                logging.info('sqs_conn.connect(queue=%r) sqshost=%r OK', queue, sqshost)
    sqs_conn = sqs_conn_dict.get((sqshost, queue))
    if sqs_conn is None:
        logging.info('sqs_conn_dict.get((sqshost=%r, queue=%r)) failed, continue', sqshost, queue)
        return
    gevent.spawn(_ics_doupload, s3_conn, bucket, sqs_conn, queue, ip, compress, timestamp, data)

def _common_upload_by_timestamp(timestamp, compress, upload_ops):
    now = int(time.time())
    logging.debug('now=%r is large enough, begin common_uploader for timestamp=%s', now, timestamp)
    dataList = LOGBLOCK_LINES.pop((timestamp), [])
    # do log filtering: (1) get access log (2) log filtering access log
    if LOG_REDUCTION:
        accessDataList = []
        accessDataListNew = []
        otherDataList = []
        for i, dataItem in enumerate(dataList):
            msgtype = int(dataItem.split('\t', 2)[1])
            if msgtype == MsgType.ACCESS_LOG:
                data = dataItem.strip('\n').split('\t')
                accessDataList.append(data)
            else:
                otherDataList.append(dataItem)
        # do filtering
        logging.debug(">>>> access log length :%r", len(accessDataList))
        if accessDataList:
            accessDataList = manager.doLogFiltering(accessDataList)
            logging.debug(">>>>> do log filtering length:%r", len(accessDataList))
        #log check EXCEPTION_LIST again
        if AgentConfig.EXCEPTION_LIST != '':
            for i,accessData in enumerate(accessDataList):
                if not isFilterMatched(accessData):
                    accessDataListNew.append(accessData)
            accessDataList=accessDataListNew;

        for i, accessDataItem in enumerate(accessDataList):
            accessDataList[i] = '\t'.join(accessDataItem) + '\n'
        accessDataList.extend(otherDataList)
        data = ''.join(accessDataList)
    else:
        data = ''.join(dataList)
    LOGBLOCK_TIMER.pop(timestamp)
    if not data:
        logging.warning('LOGBLOCK_LINES.pop((timestamp=%r), []) return NULL!', timestamp) # cid for iwss is 0
        return
    gevent.spawn(_common_doupload, compress, timestamp, data, upload_ops)

    # For Skynet integration, need convert id to l10n value in raw log,
    # then upload to stats calcuation process.
    # Requirement change: no need to convert id to l10n value.
    if PRODUCT == Product.IWSS and AgentConfig.getUploadStatsConfig():
        gevent.spawn(_common_doupload_skynet, compress, timestamp, data)

def uploader(options, event):
    """A Log Block Upload Greenlet"""
    awsid    = options.awsid
    awskey   = options.awskey
    cassa    = options.cassa
    compress = options.compress
    ip       = options.ip
    sleep    = options.sleep
    timeblock = options.timeblock
    s3host    = options.s3host

    s3_conn, sqs_conn_dict, sqs_conn_dict_lock = None, None, None
    if PRODUCT == Product.ICS:
        # S3 connection
        s3_conn = S3.AWSAuthConnection(awsid, awskey, is_secure=False, server=s3host)
        # SQS connection
        sqs_conn_dict = {}
        sqs_conn_dict_lock = gevent.coros.Semaphore()

    last_timestamp = None
    while not event.wait(sleep):
        try:
            if PRODUCT == Product.ICS:
                for timestamp, bucket, queue, sqshost in LOGBLOCK_LINES:
                    if (timestamp, bucket, queue, sqshost) not in LOGBLOCK_GREENLETS:
                        LOGBLOCK_GREENLETS[(timestamp, bucket, queue, sqshost)] = None
                        logging.info('gevent.spawn_later(MAX_LOG_FLUSHTIME=%r, _upload_by_timestamp, timestamp=%r, bucket=%r, queue=%r, sqshost=%r, ...)', MAX_LOG_FLUSHTIME, timestamp, bucket, queue, sqshost)
                        greenlet = gevent.spawn_later(MAX_LOG_FLUSHTIME, _ics_upload_by_timestamp, timestamp, bucket, queue, sqshost, awsid, awskey, s3_conn, sqs_conn_dict, sqs_conn_dict_lock, ip, compress)
                        LOGBLOCK_GREENLETS[(timestamp, bucket, queue, sqshost)] = greenlet
            elif PRODUCT == Product.IWSS or PRODUCT == Product.DEEP_EDGE:
                if PRODUCT == Product.IWSS:
                    upload_ops = iwss_upload_ops()
                elif PRODUCT == Product.DEEP_EDGE:
                    upload_ops = de_upload_ops()
                timestamps = LOGBLOCK_LINES.keys()
                if len(timestamps) > 0:
                    timestamps.sort()
                    for timestamp in timestamps:
                        length = len(LOGBLOCK_LINES[timestamp])
                        if length >= AgentConfig.UPLOAD_MAX_LINES:
                            logging.info('gevent.spawn(REACH_MAX_LINES=%r, _common_upload_by_timestamp, timestamp=%r ...)', length, timestamp)
                            gevent.spawn(_common_upload_by_timestamp, timestamp, compress, upload_ops)
                        elif abs(time.time() - LOGBLOCK_TIMER[timestamp]) > AgentConfig.UPLOAD_MAX_TIMEOUT:
                            logging.info('gevent.spawn(REACH_MAX_TIME =%r, _common_upload_by_timestamp, timestamp=%r ...)', abs(time.time() - LOGBLOCK_TIMER[timestamp]), timestamp)
                            gevent.spawn(_common_upload_by_timestamp, timestamp, compress, upload_ops)
                else:
                    time.sleep(MAX_UPLOAD_WAIT)
        except Exception, e:
            logging.exception('uploader get a exception %s', e)
            continue
    logging.info('uploader greenlet=%s exit', gevent.getcurrent())

def failed_uploader(options, event):
    """A Log Block Upload Greenlet"""
    awsid    = options.awsid
    awskey   = options.awskey
    cassa    = options.cassa
    compress = options.compress
    ip       = options.ip
    sleep    = options.sleep
    timeblock = options.timeblock
    s3host    = options.s3host

    s3_conn, sqs_conn_dict, sqs_conn_dict_lock = None, None, None
    if PRODUCT == Product.ICS:
        # S3 connection
        s3_conn = S3.AWSAuthConnection(awsid, awskey, is_secure=False, server=s3host)
        # SQS connection
        sqs_conn_dict = {}
        sqs_conn_dict_lock = gevent.coros.Semaphore()
    elif PRODUCT == Product.IWSS:
        pass
    elif PRODUCT == Product.DEEP_EDGE:
        pass
    else:
        logging.error('failed_loader error: product=%s', PRODUCT)
        sys.exit(-1)

    filename = None
    last_timestamp = None
    while not event.wait(sleep):
        try:
            if PRODUCT == Product.ICS:
                timestamp, bucket, queue, sqshost, ip, data = LOGBLOCK_FAILED.popleft()
                if (sqshost, queue) not in sqs_conn_dict:
                    with sqs_conn_dict_lock:
                        if (sqshost, queue) not in sqs_conn_dict:
                            logging.info('sqs_conn.connect(queue=%r) sqshost=%r', queue, sqshost)
                            sqs_conn = SQS.AWSAuthConnection(awsid, awskey, False, server=sqshost)
                            sqs_conn.connect(queue)
                            sqs_conn_dict[(sqshost, queue)] = sqs_conn
                            logging.info('sqs_conn.connect(queue=%r) sqshost=%r OK', queue, sqshost)
                sqs_conn = sqs_conn_dict.get((sqshost, queue))

                key = '%s/%s_%s.log' % (BUCKET_KEYNAME_PREFIX, timestamp, ip)
                logging.info('failed_uploader S3 bucket put key=%r data size=%r', key, len(data))
                response = s3_conn.put(bucket, key, S3.S3Object(data))
                if response.message == '200 OK':
                    logging.info('S3 bucket put response.message=%r', response.message)
                else:
                    logging.error('S3 bucket put failed: response.message=%r', response.message)
                    LOGBLOCK_FAILED.appendleft((timestamp, bucket, queue, sqshost, ip, data))
                    time.sleep(MAX_UPLOAD_FAILWAIT)
                    continue
                # send SQS message to logwriter server
                data = {'timestamp':timestamp, 'ip':ip, 'bucket':bucket, 'key':key}
                logging.info('SQS queue write message(data=%s)', data)
                response = sqs_conn.send_message(queue, json.dumps(data))
                logging.info('SQS queue write message end, response.message=%r', response.message)
            elif PRODUCT == Product.IWSS:
                timestamp, data = FailedTimestampFile.read()
                if timestamp == 0:
                    time.sleep(6*MAX_UPLOAD_FAILWAIT)
                    continue
                filename = ''
                postdata = {}
                logging.info('failed_uploader upload again(timestamp=%d, datasize=%d)', timestamp, len(data))
                if STORAGE_TYPE == DeployMode.ALL_IN_ONE:
                    filename = os.path.join(AgentConfig.UPLOAD_DATA_DIR, ('%d_%s' % (timestamp, str(uuid.uuid1()))) )
                    with open(filename, 'wb') as fp:
                        fp.write(data)
                        fp.close()
                    postdata = {'ts':timestamp, 'location':filename}
                elif STORAGE_TYPE == DeployMode.STAND_ALONE:
                    if compress:
                        compressed_data = base64.encodestring(zlib.compress(data, int(compress)))
                        postdata = {'ts':timestamp, 'zip':1, 'content':compressed_data}
                    else:
                        postdata = {'ts':timestamp, 'zip':0, 'content':data}

                postdata = json.dumps(postdata)
                logUploadURL = ''.join((AgentConfig.PROTOCOL, '://', LOG_SERVER, '/log_upload'))

                auth_headers = AgentConfig.getAuthHeader()
                while 1:
                    try:
                        request = urllib2.Request(logUploadURL, postdata, auth_headers)
                        fd = urllib2.urlopen(request, timeout = 1800)
                        responseCode = fd.getcode()
                        if responseCode == 200:
                            logging.info("failed_upload successfully: response.code=%d", responseCode)
                            FailedTimestampFile.delete(timestamp)
                            if not AgentConfig.UPLOAD_STATUS:
                                IwssSendEventLog.send(IwssSendEventLog.EVT_LOG_UPLOAD_RECOVER)
                                AgentConfig.UPLOAD_STATUS = True
                                Configer.set(AgentConfig.SYSEVENT_CONFIGFILE, 'UPLOAD_STATUS', 'status', AgentConfig.UPLOAD_STATUS)
                            break
                        else:
                            logging.info("iwss_failed_upload failed: response.code=%d", responseCode)
                    except urllib2.URLError as e:
                        logging.error('iwss_failed_upload log block error: %s', e)
                    except Exception as e:
                        logging.exception("iwss_failed_upload failed: unknown exception %s", e)
                    time.sleep(6*MAX_UPLOAD_FAILWAIT)
            elif PRODUCT == Product.DEEP_EDGE:
                pass
        except IndexError as e:
            time.sleep(MAX_UPLOAD_FAILWAIT)
        except Exception, e:
            logging.exception('failed_uploader get a exception %s', e)
            if PRODUCT == Product.ICS:
                LOGBLOCK_FAILED.appendleft((timestamp, bucket, queue, sqshost, ip, data))
            elif PRODUCT == Product.IWSS:
                if os.path.isfile(filename):
                    os.remove(filename)
            elif PRODUCT == Product.DEEP_EDGE:
                pass
    logging.info('uploader greenlet=%s exit', gevent.getcurrent())

class ProcessLogReader(object):
    """Scan Process Log Line Reader, with rotating support"""

    def __init__(self, taskname, product):
        self.product = product
        self.taskname = taskname
        self.cleanfile()                   # delete the log file except the current file

        self.filename = _getfilename(self.taskname, self.product)
        self.nextday_filename = _getnextfilename(self.filename)  # new filename of next day
        self.readline_time = int(time.time())    # lastime of read out data
        self.fp = None                     # current fp
        self.__line = ''                   # current line cache, if we dont read to EOL

    def cleanfile(self):
        if PRODUCT == Product.ICS:
            pass
        elif PRODUCT == Product.IWSS:
            filename  = _getfilename(self.taskname, self.product)
            filenames = os.popen( 'ls %s*' %(filename[0:-8]) ).read().splitlines()
            if filename in filenames:
                filenames.remove(filename)
            for name in filenames:
                os.remove(name)
        elif PRODUCT == Product.DeepEdge:
            pass

    def readline(self):
        """read a line
        if read a EOL, return a whole line;
        if read a uncompleted line, cache it to self.__line and return ''
        if filename has been rotated, it will read new from new file
        if next day filename is detected, it will switch to next day filename
        """
        if self.fp is None:
            try:
                # open current filname
                self.fp = open(self.filename, 'rb')
                os.system('touch %s' % self.filename)
            except EnvironmentError, e:
                # open fail, adjust latest filename, then return '' to sleep
                logging.debug('open(filename=%r, "rb") fail: %s, try update current self.filename', self.filename, e)
                self.filename = _getfilename(self.taskname, self.product)
                self.nextday_filename = _getnextfilename(self.filename)
                return ''
        # try read a new line
        line = self.fp.readline()
        if line == '':
            # next day filename exsits?
            if os.path.isfile(self.nextday_filename):# swtich new file
                logging.info('ProcessLogReader swtich nextday_filename=%r', self.nextday_filename)
                self.fp.close()
                # delete the log files that have been read
                self.cleanfile()
                self.fp = None
                self.filename = _getfilename(self.taskname, self.product)
                self.nextday_filename = _getnextfilename(self.filename)
                self.__line = ''
                return self.readline()
            # current filename been rename
            elif not os.path.exists(self.filename):
                # just return '' and sleep
                logging.info('cannot find current filename=%r, maybe rotating', self.filename)
                return ''
            # filename is rotated
            elif int(os.path.getmtime(self.filename)) > self.readline_time:
                # rotate read line
                self.fp.close()
                # delete the log files that have been read
                self.cleanfile()
                self.fp = None
                self.__line = ''
                logging.info('LineReader detect rotating, reopen %r', self.filename)
                time.sleep(1)
                return self.readline()
            else:
                return ''
        # read a whole
        elif line[-1] == '\n':
            self.readline_time = int(time.time()) # set readline_time
            # return line
            if self.__line:
                line = self.__line + line
                self.__line = ''
                return line
            else:
                return line
        # read does get a EOLF
        else:
            # just cache it to __line and sleep
            self.readline_time = int(time.time())
            self.__line += line
            return ''

    def close(self):
        if self.fp is not None:
            try:
                self.fp.close()
            except Exception, e:
                logging.error('ProcessLogReader.close try close self.fp failed:%s', e)
            finally:
                self.fp = None

def _getiwssfilename(taskname):
    filename_prefix = "%s.%s.*" % (IWSS_LOG_FILENAME_PREFIX, taskname)
    if AgentConfig.MIGRATION == 1:
        filename_prefix = "%s.%s.*" % (IWSS_LOG_FILENAME_PREFIX_MIGRATION, taskname)

    targetfile = ""
    filenamelist = glob.glob(filename_prefix)
    if filenamelist:
        files = [(os.path.getmtime(filename), filename)for filename in filenamelist]
        files.sort()
        targetfile = files[0][1]
        if AgentConfig.MIGRATION == 1:
            (filepath, filename) = os.path.split(targetfile)
            AgentConfig.MIGRATION_IP = filename.split('_')[-1]
            logging.info(AgentConfig.MIGRATION_IP)

    return targetfile

class IwssProcessLogReader(object):
    """ProcessLogReader has bugs when changing system time"""
    """So i create this function for Iwss. (Figo)         """

    def __init__(self, taskname, product):
        self.product  = product
        self.taskname = taskname
        self.fp       = None                           # current fp
        self.__line   = ""
        self.RBACdict = {}

    def getRBACInfo(self, userNameList, **kwargs):
        if not userNameList or not isinstance(userNameList,list):
            logging.info('getRBAC bitmap user %r', userNameList)
            return {'error': 'parameter error'}

        reTry = 0
        postdata = json.dumps(userNameList)
        auth_headers = None
        while reTry < 3:
            try:
                request = urllib2.Request('http://127.0.0.1:6012/rbac_query', postdata)
                fd = urllib2.urlopen(request, timeout = 1800)
                responseCode = fd.getcode()
                if responseCode == 200:
                    data = fd.read()
                    return json.loads(data)
                else:
                    reTry += 1
                    logging.info('getRBAC bitmap responseCode :%r', responseCode)
            except urllib2.URLError as e:
                reTry += 1
                logging.error('get RBAC bitmap urllib2 error:%r', e)
            except Exception as e:
                reTry += 1
                logging.error('get RBAC bitmap error:%r', e)

        logging.info('reTry count:%r', reTry)
        return {}

    def readline(self):
        """read a line
        """
        if self.fp is None:
            try:
                # obtain an report file name, then open
                self.filename = _getiwssfilename(self.taskname)
                if self.filename:
                    filenameing = self.filename + ".ing"
                    os.renames(self.filename, filenameing)
                    self.filename = filenameing
                if not self.filename:
                    return '', self.RBACdict
                #with open(self.filename, 'rb+') as tmpFp:
                #    data = tmpFp.read()
                #    lines = data.split('\n')
                #    userNameList = set()
                #    for line in lines:
                #        if line:
                #            userNameList.add(line.split('\t')[0])
                #    # from webservice getting RBAC information
                #    userNameList = list(userNameList)
                #    self.RBACdict = self.getRBACInfo(userNameList)
                self.fp = open(self.filename, 'rb')

            except EnvironmentError, e:
                # open fail, adjust latest filename, then return '' to sleep
                logging.info('open(filename=%r, "rb") fail: %s, try update current self.filename', self.filename, e)
                #logging.error(traceback.format_exc())
                return '', self.RBACdict
        # try read a new line
        line = self.fp.readline()
        if line == '':
            # finish reading current file
            self.fp.close()
            self.fp = None
            os.remove(self.filename)
            # make sure the current file is removed
            if os.path.exists(self.filename):
                os.remove(self.filename)
             # clean all RBAC information
            self.RBACdict = {}
            return '', self.RBACdict
        # read a whole
        elif line[-1] == '\n':
            # return line
            if self.__line:
                line = self.__line + line
                self.__line = ''
                return line, self.RBACdict
            else:
                return line, self.RBACdict
        # read does get a EOLF
        else:
            self.__line += line
            return '', self.RBACdict
    def close(self):
        if self.fp is not None:
            try:
                self.fp.close()
                os.remove(self.filename)
            except Exception, e:
                logging.error('IwssProcessLogReader.close try close self.fp failed:%s', e)
            finally:
                self.fp = None

def DEProcessLogReader(taskname, timeblock):
    if taskname == DE_LOG_NAME.DE_VIOLATION_LOG:
        delog = DEViolationLog()
    elif taskname == DE_LOG_NAME.DE_ACCESS_LOG:
        delog = DEAccessLog()
    elif taskname == DE_LOG_NAME.DE_APPBAND_LOG:
        delog = DEAppbandLog()
    while 1:
        if not delog.openLog():
            gevent.sleep(2)
            continue
        logconv = DeepEdgeLogConverter()
        while 1:
            line = delog.readline()
            if not line:
                # This sleep is very important
                gevent.sleep(1)
                continue
            if taskname == DE_LOG_NAME.DE_VIOLATION_LOG:
                cid, timestamp, line = logconv.convert_violation(line)
            elif taskname == DE_LOG_NAME.DE_ACCESS_LOG:
                cid, timestamp, line = logconv.convert_access(line)
            elif taskname == DE_LOG_NAME.DE_APPBAND_LOG:
                cid, timestamp, line = logconv.convert_appband(line)
            timestamp_shift = int(timestamp) / timeblock * timeblock
            if line:
                LOGBLOCK_LINES[(timestamp_shift)].append(line)
            delog.dumpCurrentBookmarkIntoFile()
#            logging.debug('Read DeepEdge log line %s', line)

def reader(options, semaphore, taskname):
    """A thread read line from log file and send log line blocks to uploader"""
    sleep      = options.sleep
    timeblock  = options.timeblock
    s3host     = options.s3host
    bucket     = options.bucket
    sqshost    = options.sqshost
    queue      = options.queue
    cassa      = options.cassa

    logging.info('logwriter-client(product=%s, taskname=%r) new a ProcessLogReader', PRODUCT, taskname)

    logreader, log_converter = None, None
    if PRODUCT == Product.ICS:
        logreader = ProcessLogReader(taskname, PRODUCT)
        log_converter = ICSLogConverter()
    elif PRODUCT == Product.IWSS:
        logreader = IwssProcessLogReader(taskname, PRODUCT)
        log_converter = IwssLogConverter()
    elif PRODUCT == Product.DEEP_EDGE:
        DEProcessLogReader(taskname, timeblock)
        return

    while 1:
        try:
            # get the current timestamp
            if PRODUCT == Product.IWSS:
                line, RBACdict  = logreader.readline()
            else:
                line = logreader.readline()
            if not line:
                # get a empty line, just sleep
                if semaphore and semaphore.acquire(timeout=sleep):
                    logging.info('reader(product=%s, taskname=%r) greenlet=%r acquire stop semaphore ok, exit.', PRODUCT, taskname, gevent.getcurrent())
                    logreader.close()
                    return
                else:
                    #logging.info(">>>>>> will not wait %r", sleep)
                    time.sleep(5)
                    continue
            # get a log entry, just cache it to blockio
            logging.debug('logwriter-client(product=%s, taskname=%r) read a line length=%s', PRODUCT ,taskname,len(line))
            if PRODUCT == Product.ICS:
                # convert ics access log to the unified log
                cid, timestamp, line = log_converter.convert(line)
                timestamp_shift = int(timestamp) / timeblock * timeblock
                logging.debug('ics_reader get timestamp=%r line length=%r timestamp_shift=%r', timestamp, len(line), timestamp_shift)
                homedc = _query_homedc(cid, cassa)
                s3_bucket = homedc.get('S3') or bucket
                sqs_queue = homedc.get('SQS') or queue
                sqs_host  = homedc.get('SQSHOST') or sqshost
                LOGBLOCK_LINES[(timestamp_shift, s3_bucket, sqs_queue, sqs_host)].append(line)
            if PRODUCT == Product.IWSS:
                if taskname == IWSS_LOG_NAME.IWSS_APPLICATION_LOG:
                    cid, timestamp, line = log_converter.convert_app(line, RBACdict)
                elif taskname == IWSS_LOG_NAME.IWSS_ACCESS_LOG:
                    cid, timestamp, line = log_converter.convert_access(line, RBACdict)
                elif taskname == IWSS_LOG_NAME.IWSS_VIOLATION_LOG:
                    cid, timestamp, line = log_converter.convert_violation(line, RBACdict)
                elif taskname == IWSS_LOG_NAME.IWSS_CCCA_LOG:
                    cid, timestamp, line = log_converter.convert_ccca(line, RBACdict)
                timestamp_shift = int(timestamp) / timeblock * timeblock
                logging.debug('iwss_reader get timestamp=%r line length=%r timestamp_shift=%r', timestamp, len(line), timestamp_shift)
                if line:
                    if not LOGBLOCK_TIMER.has_key(timestamp_shift):

                        if AgentConfig.EXCEPTION_LIST != '':
                            line_list = line.split('\t')
                            if isFilterMatched(line_list):
                                continue
                                
                        logging.debug("MAX QUEUE is [%d], current QUEUE is [%d]", AgentConfig.UPLOAD_MAX_QUEUE ,len(LOGBLOCK_TIMER))
                        while len(LOGBLOCK_TIMER) > AgentConfig.UPLOAD_MAX_QUEUE :
                            time.sleep(5)
                        if not LOGBLOCK_TIMER.has_key(timestamp_shift):
                            #if taskname == 'access' and LOG_REDUCTION:
                                # for access log, log reduction have been sleep 60s
                            #    LOGBLOCK_TIMER[timestamp_shift] = int(time.time()-60)
                            #else:
                            LOGBLOCK_TIMER[timestamp_shift] = int(time.time())

                    if AgentConfig.EXCEPTION_LIST == '':
                        logging.debug('No Log Filter !')
                        LOGBLOCK_LINES[(timestamp_shift)].append(line)
                    else:
                        line_list = line.split('\t')
                        if isFilterMatched(line_list):
                            pass
                        else:
                            logging.debug('%s',line_list)
                            LOGBLOCK_LINES[(timestamp_shift)].append(line)
        except IOError, e:
            # some filesystem error, exit
            logging.exception('Log line parse ValueError: error=%s', e)
        except (ValueError, IndexError), e:
            # parse log entry, ignore it and continue
            logging.exception('Log line parse ValueError: error=%s', e)
        except Exception, e:
            # unkown error, log it and continue
            logging.exception('Exception: error=%s', e)

def isFilterMatched(line_list):
    except_list = eval(AgentConfig.EXCEPTION_LIST)

    logging.debug('%s',except_list[0]['type'])

    for rule in except_list:
        typeMatched = matched_msgtype(rule['type'],line_list[1])
        logging.debug('%s %s',rule['type'],line_list[1])
        logging.debug('msg matched ? %s',typeMatched)
        if not typeMatched :
            continue
        if rule['user'] == '' :
            userMatched = True
            if line_list[19] == '' :
                domMatched = False
            else:
                domMatched  = wildcard_match(rule['dom'],line_list[19])
            logging.debug('%s %s',rule['dom'],line_list[19])
            logging.debug('dom matched ? %s',domMatched)
        else:
            domMatched  = True
            userMatched = wildcard_match(rule['user'],line_list[6])
            logging.debug('%s %s',rule['user'],line_list[6])
            logging.debug('user matched ? %s',userMatched)
        if typeMatched and userMatched and domMatched :
            logging.debug('cloud_filter_rule matched !')
            return True
    return False


def matched_msgtype(msgtype,msgtype_filed):
    type = int(msgtype)
    field = int(msgtype_filed)
    if field in MSGTYPE_LIST[type]:
        return True
    else:
        return False

def wildcard_match(pattern,mystr):

	#For SEGTT 312546
    logging.debug('wildcard_match : Full string is  [%s], pattern is [%s]',mystr,pattern)

    if fnmatch.fnmatch(mystr, pattern):
        return True
    else:
        return False


def monitor(options, event):
    greenlets = {}
    while not event.wait(options.sleep):
        if PRODUCT == Product.IWSS:
            # logname -> (greenlet, event), type {str:(gevent.Greenlet, gevent.coros.Semaphore)}
            lognames = [IWSS_LOG_NAME.IWSS_CCCA_LOG, IWSS_LOG_NAME.IWSS_VIOLATION_LOG, IWSS_LOG_NAME.IWSS_ACCESS_LOG, IWSS_LOG_NAME.IWSS_APPLICATION_LOG]
            logging.debug('monitor lognames=%s', lognames)
            for logname in lognames:
                logging.info('gevent.spawn(iwss_reader, options, semaphore, logname=%s)', logname)
                # start a reader greenlet to follow the 'logname' log
                semaphore = gevent.coros.Semaphore()
                semaphore.acquire()
                greenlet = gevent.spawn(reader, options, semaphore, logname)
                greenlets[logname] = greenlet
	    gevent.joinall([greenlets[logname] for logname in greenlets])
        elif PRODUCT == Product.ICS:
            # pid -> (greenlet, event), type {str:(gevent.Greenlet, gevent.coros.Semaphore)}
            pids = _getpids()
            logging.debug('monitor _getpids() return pids=%s', pids)
            for pid in set(pids)-set(greenlets):
                logging.info('current pid=%r not in dict greenlets=%s, try spawn', pid, set(greenlets))
                logging.info('gevent.spawn(ics_reader, options, semaphore, pid=%r)', pid)
                # start a reader greenlet to follow the `pid` log
                semaphore = gevent.coros.Semaphore()
                semaphore.acquire()
                greenlet = gevent.spawn(reader, options, semaphore, pid)
                greenlets[pid] = (greenlet, semaphore)
            for pid in set(greenlets)-set(pids):
                logging.info('old pid=%r not in dict greenlets=%s, try kill', pid, set(greenlets))
                try:
                    # try stop the dead `pid` reader greenlet
                    greenlet, semaphore = greenlets.pop(pid)
                    semaphore.release()
                except Exception, e:
                    logging.info('try kill old pid=%r not in greenlets failed:%s', e)
        # monitor for DeepEdge
        elif PRODUCT == Product.DEEP_EDGE:
            lognames = [DE_LOG_NAME.DE_VIOLATION_LOG, DE_LOG_NAME.DE_ACCESS_LOG, DE_LOG_NAME.DE_APPBAND_LOG]
            logging.debug('monitor lognames=%s', lognames)
            for logname in lognames:
                logging.info('gevent.spawn(de_reader, options, semaphore, logname=%s)', logname)
                # start a reader greenlet to follow the 'logname' log
                greenlet = gevent.spawn(reader, options, None, logname)
                greenlets[logname] = greenlet
            gevent.joinall([greenlets[logname] for logname in greenlets])
        else:
            logging.error('log agent does not support the current product(%s)', PRODUCT)
            sys.exit(-1)

    for _, event in greenlets.itervalues():
        event.set()

def checkargs(options, args):
    """check options/args, raise AssertionError if not vaild"""
    #assert re.match(r'\w+', options.awsid)
    #assert re.match(r'[\w/]+', options.awskey)
    #assert re.match(r'\w+', options.s3host)
    #assert re.match(r'\w+', options.bucket)
    #assert re.match(r'\w+', options.sqshost)
    #assert re.match(r'\w+', options.queue)
    assert re.match(r'\d+,\d+,\d+', options.gc)
    assert 1 <= options.timeblock <= 3600
    assert 1 <= options.sleep <= 600
    assert 0 <= options.compress <= 9
    #assert 0 == os.system('which ntpdate')

def main():
    # check whether is migration starting
    if ( len(sys.argv) > 1 ):
        if sys.argv[1] == 'migration':
            AgentConfig.MIGRATION = 1;

    # set conguration
    AgentConfig.init()

    global PRODUCT, STORAGE_TYPE, LOG_SERVER, LOG_REDUCTION
    PRODUCT                 = AgentConfig.PRODUCT
    STORAGE_TYPE            = AgentConfig.DEPLOY_MODE
    LOG_SERVER              = AgentConfig.LOG_SERVER_ADDR + ':' + str(AgentConfig.LOG_SERVER_PORT)
    compress_state          = AgentConfig.COMPRESS_STATE
    LOG_REDUCTION           = AgentConfig.LOG_REDUCTION
    logging.debug(">>>>> LOG_REDUCTION:%r, AgentConfig.LOG_REDUCTION:%r",LOG_REDUCTION,AgentConfig.LOG_REDUCTION)

    DEFAULT_AWSID     = ''
    DEFAULT_AWSKEY    = ''
    DEFAULT_S3HOST    = ''
    DEFAULT_BUCKET    = ''
    DEFAULT_SQSHOST   = ''
    DEFAULT_SQS       = ''
    DEFAULT_CASSA     = ''
    DEFAULT_TIMEBLCOK = 60
    DEFAULT_SLEEP     = 1
    DEFAULT_COMPRESS  = compress_state
    DEFAULT_GC        = '70,10,10'
    DEFAULT_IP        = ''
    DEFAULT_IFNAME    = 'eth0'

    parser = optparse.OptionParser(usage='usage: %prog [options]')
    parser.add_option('--awsid',     dest='awsid',     type='string', default=DEFAULT_AWSID,     help='aws_access_key_id')
    parser.add_option('--awskey',    dest='awskey',    type='string', default=DEFAULT_AWSKEY,    help='aws_secret_access_key')
    parser.add_option('--s3host',    dest='s3host',    type='string', default=DEFAULT_S3HOST,    help='S3 host server')
    parser.add_option('--bucket',    dest='bucket',    type='string', default=DEFAULT_BUCKET,    help='S3 bucket name')
    parser.add_option('--sqshost',   dest='sqshost',   type='string', default=DEFAULT_SQSHOST,   help='SQS host server')
    parser.add_option('--queue',     dest='queue',     type='string', default=DEFAULT_SQS,       help='SQS queue name')
    parser.add_option('--cassa',     dest='cassa',     type='string', default=DEFAULT_CASSA,     help='CASSA homedc address')
    parser.add_option('--timeblock', dest='timeblock', type='int',    default=DEFAULT_TIMEBLCOK, help='S3 put timeblock')
    parser.add_option('--sleep',     dest='sleep',     type='int',    default=DEFAULT_SLEEP,     help='sleep seconds after get empty line')
    parser.add_option('--compress',  dest='compress',  type='int',    default=DEFAULT_COMPRESS,  help='log compress level')
    parser.add_option('--ip',        dest='ip',        type='string', default=DEFAULT_IP,        help='a postfix of bucket key postfix, usually IP address')
    parser.add_option('--ifname',    dest='ifname',    type='string', default=DEFAULT_IFNAME,    help='default interface name, usually `eth0`')
    parser.add_option('--gc',        dest='gc',        type='string', default=DEFAULT_GC,        help='gc threshold')

    options, args = parser.parse_args()

    if not options.ip:
        # For ICS
        if PRODUCT == Product.ICS:
            options.ip = _getip('eth0')

    logging.info('options=%s, args=%s', options, args)

    checkargs(options, args)

    gc.set_threshold(*[int(x) for x in options.gc.split(',')])

    logging.info('Current Product:' + str(PRODUCT) +', Storage_type: ' + str(STORAGE_TYPE) + ', Log_server: ' + str(LOG_SERVER))
    events = []
    try:
        targets = [uploader, failed_uploader, monitor]
        events = [gevent.event.Event() for i in xrange(len(targets))]
        greenlets = [gevent.spawn(target, options, event) for target, event in zip(targets, events)]
        gevent.joinall(greenlets)
    except (KeyboardInterrupt, SystemExit):
        logging.info('receive exit exception, try to set exit events')
        for event in events:
            event.set()

if __name__ == '__main__':
    main()
