#!/bin/sh

MODULE_NAME="migration"

MIG_PKG_HOME="/usr/iwss/migration/migration_packages"
IWSS_HOME="/var/iwss"
MIG_PATH="$IWSS_HOME/migration"
MIG_TMP_PATH="$MIG_PATH/tmp"
MIG_EXPORT_PATH="$MIG_PATH/export"
MIG_SCHEDULE_EXPORT_PATH="$MIG_PATH/schedule"
CDT_CONVERTED_PATH="$MIG_PATH"

# Log and configure files
PRODUCT_INFO_FILE="$MIG_TMP_PATH/ProductInfo.xml"
CDT_CONVERTED_PACKAGE="$CDT_CONVERTED_PATH/IWSS_Tmp_Config.tar"
IMPORT_UI_LOG="/tmp/import_result.txt"
EXPORT_UI_LOG="/tmp/export_result.txt"
IMPORT_UI_POLLING_FLAG="/tmp/import_polling_progress_start"
MIG_RES_FILE="$IWSS_HOME/resource/config_backup.txt"
DEBUG_LOG_FILE="/tmp/migration_sh.log"
IWSS_INTSCAN_INI="/etc/iscan/intscan.ini"
MIGRATION_CONFIG_INI="/usr/iwss/migration/migration.ini"
MIG_LOG_FILE="/etc/iscan/log/migration.log.`date +\"%Y%m%d\"`.0001"
CACHE_CFG="/var/iwss/iwssCache.ini"
CACHE_HELPER="/usr/iwss/AdminUI/cache_helper.sh"
MIG_SCHEDULE_CLEAN_BACKUP=no
MIG_SCHEDULE_LATEST_BACKUP_NUM=1
#0: no log; 1: common log; 2: debug log
MIG_LOG_LEVEL="2"

# Binary tools
PROMOTE_PERMISSION="/usr/iwss/AdminUI/uihelper"
MIG_MAIN_TOOL="/usr/iwss/migration/migration -L $MIG_LOG_LEVEL -V $MIG_LOG_FILE "
INICONFIG_TOOL="/usr/bin/iniconfig"
PSQL_CLIENT="/usr/iwss/PostgreSQL/bin/psql"
CDT_PKG_COVERT_TOOL="/usr/iwss/migration/migration_scripts/ConvertCDTPkg.sh"
LDAP_SYNC_TOOL="/usr/iwss/commonldap/LdapSyncTool.sh"

# Return value of migration tool
ERR_PARAMETER_INVALID=100          # Invalid parameter
ERR_ACCOUNT_INVALID=101            # Invalid account
ERR_IMPORT_PKG_PERM=102            # Inavlid import package
ERR_MUTIPLE_INSTANCE_RUNNING=103   # Alread running
ERR_DB_CONNECT_FAILURE=104         # Fail to DB connection
ERR_INVAID_BUILD=105

ERR_INVALID_IMPORT_PKG=153
ERR_INVALID_VERSION=154
ERR_PATCH_RUNNING=155

PKG_TYPE_NOTSUPPORT=100
PKG_TYPE_IWSS=0
PKG_TYPE_IWSVA31=1
PKG_TYPE_IWSVA50=2
PKG_TYPE_IWSVA51=3
PKG_TYPE_IWSVA51SP1=4

MIG_UPGRADE_VA="true"

F_usage()
{
	echo "Usage:"
	echo "$0 -C <CheckPackageName>"
	echo "$0 -E <ExportPackageName> [--PSQLParameter=***]"
	echo "$0 -I <ImportPackageName> [--FULL|PARTIAL|SYNC] [--NotApply] [--DBPolicy|ConfigFile] [--PSQLParameter=***]"
	echo "$0 -S"
	echo "$0 -D <DeletePackageName>"
	echo ""
}

F_get_default_values()
{
  if [ -f "$MIGRATION_CONFIG_INI" ]; then

	option_value=`$INICONFIG_TOOL -r -f "$MIGRATION_CONFIG_INI" -s "common"  -o "WorkDir"`
	[ -n "$option_value" ] && MIG_PATH=$option_value

	option_value=`$INICONFIG_TOOL -r -f "$MIGRATION_CONFIG_INI" -s "common" -o "DebugLog"`
	[ -n "$option_value" ] && MIG_LOG_LEVEL=$option_value

	option_value=`$INICONFIG_TOOL -r -f "$MIGRATION_CONFIG_INI" -s "common" -o "LogFile"`
	[ -n "$option_value" ] && MIG_LOG_FILE=$option_value

	option_value=`$INICONFIG_TOOL -r -f "$MIGRATION_CONFIG_INI" -s "import" -o "TmpDir"`
	[ -n "$option_value" ] && MIG_TMP_PATH=$option_value

	option_value=`$INICONFIG_TOOL -r -f "$MIGRATION_CONFIG_INI" -s "import" -o "MigrateFrom"`
	[ -n "$option_value" ] && MIGRATE_FROM=$option_value

	option_value=`$INICONFIG_TOOL -r -f "$MIGRATION_CONFIG_INI" -s "export" -o "ExportDir"`
	[ -n "$option_value" ] && MIG_EXPORT_PATH=$option_value

	option_value=`$INICONFIG_TOOL -r -f "$MIGRATION_CONFIG_INI" -s "schedule" -o "CleanBackup"`
	[ -n "$option_value" ] && MIG_SCHEDULE_CLEAN_BACKUP=$option_value

	option_value=`$INICONFIG_TOOL -r -f "$MIGRATION_CONFIG_INI" -s "schedule" -o "LatestBackupNum"`
	[ -n "$option_value" ] && MIG_SCHEDULE_LATEST_BACKUP_NUM=$option_value

  fi

  # Apend the date to the log file
  current_date=`date +"%Y%m%d"`
  MIG_LOG_FILE="$MIG_LOG_FILE.$current_date.0001"
}

F_check_user()
{
	_user=$1
	_group=$2
	_cur_user=`id -un`
	_cur_group=`id -gn`
	if [ "$_cur_user" != "$_user" ]; then
		return 1
	fi
	if [ "$_cur_group" != "$_group" ]; then
		return 1
	fi
	return 0
}

F_UI_polling_stop()
{
	# touch a file to notify UI start to polling.
	if [ -f "$IMPORT_UI_POLLING_FLAG" ] ; then
		rm $IMPORT_UI_POLLING_FLAG
	fi
}

F_UI_polling_start()
{
	# touch a file to notify UI start to polling.
	touch $IMPORT_UI_POLLING_FLAG
}

F_UI_not_import_partially_list()
{
	#for SEG TT 324669
    env_lang=`$INICONFIG_TOOL -r -f "$IWSS_HOME/intscan.ini" -s "Common" -o "lang"`
    if [ $env_lang == "ja" ] ; then 
        F_UI_log "$MIG_RES_FILE" "not_import" "list06" "NotImported"
    else
		F_UI_log "$MIG_RES_FILE" "not_import" "list01" "NotImported"
		F_UI_log "$MIG_RES_FILE" "not_import" "list02" "NotImported"
		F_UI_log "$MIG_RES_FILE" "not_import" "list03" "NotImported"
		F_UI_log "$MIG_RES_FILE" "not_import" "list04" "NotImported"
		#remove HTTPS and Applet re-sign certifications in config_backup_result.jsp
		#iwsva6.0 sp1 have do root ca migration
		#F_UI_log "$MIG_RES_FILE" "not_import" "list05" "NotImported"

		if [ "$import_type" = "--PARTIAL" ] ; then
			F_UI_log "$MIG_RES_FILE" "not_import_partially" "list01" "NotImported"
			F_UI_log "$MIG_RES_FILE" "not_import_partially" "list02" "NotImported"
		fi

		if [ "$MIG_UPGRADE_VA" != "false" ] ; then
			#F_UI_log "$MIG_RES_FILE" "not_import_upgrade" "list01" "NotImported"
			F_UI_log "$MIG_RES_FILE" "not_import_upgrade" "list02" "NotImported"
		fi
	fi
}

F_log()
{
	echo "$1"
	if [ "$DEBUG_LOG_FILE" != "" ]; then
		echo "`date "+%Y/%m/%d %H:%M:%S"` [$mig_action] $1" >> $DEBUG_LOG_FILE
	fi
}

F_UI_log()
{
	_res_file=$1
	_res_sec=$2
	_res_key=$3
	_prefix=$4
	_extra_str=$5

	if [ ! -f "$_res_file" ]; then
		return 1
	fi
	_res_value=`$INICONFIG_TOOL -r -f $_res_file -s $_res_sec -o $_res_key`

	if [ -n "$_extra_str" ]; then
		_res_value=$_res_value$_extra_str
	fi

	if [ "$DEBUGLOG" = "1" ]; then
		echo "[UI_LOG] [[$_prefix]] $_res_value"
	fi
	if [ "$MODULE" = "Import" ]; then
		echo "[[$_prefix]] $_res_value" >> $IMPORT_UI_LOG
	elif [ "$MODULE" = "Export" ]; then
		echo "[[$_prefix]] $_res_value" >> $EXPORT_UI_LOG
	fi
	return 0
}

###
F_IsMultipleProcess()
{
  RET="true"

  # First Check for Multiple Import/Export Running
  if [ -f "$IWSS_HOME/$MODULE_NAME.pid" ] ; then
	_previous_pid=`cat $IWSS_HOME/$MODULE_NAME.pid`
  fi

  if [ -z "$_previous_pid" ] ; then
	# No other Import/Export is running
	RET="false"
  else
	_previous_pidinfo=`ps aux | grep "$_previous_pid" | grep -v 'grep' | grep -v '\<defunct\>' | grep "$MODULE_NAME"`
	if [ -z "$_previous_pidinfo" ] ; then
		# Second Check for Import/Export Running ####
		if [ -f "$IWSS_HOME/$MODULE_NAME.pid" ] ; then
		_previous_pid=`cat $IWSS_HOME/$MODULE_NAME.pid`
		fi
		if [ -z "$_previous_pid" ] ; then
		# No other Import/Export is running
		RET="false"
		else
			_previous_pidinfo=`ps aux | grep "$_previous_pid" | grep -v 'grep' | grep -v '\<defunct\>' | grep "$MODULE_NAME"`

			if [ -z "$_previous_pidinfo" ] ; then
				#No other Import/Export is running
				RET="false"
			fi
		fi
	fi
  fi

  printf "%s\n" "$RET"
  return 0
}

F_verify()
{
  F_log "F_verify() in $MIGRATE_FROM"

  _product=$1
  _version=$2
  _language=$3
  _platform=$4

  if [ -z "$MIGRATE_FROM" ]; then
	# pass verify
	return 0
  fi

  _verify_count=`echo $MIGRATE_FROM | awk 'BEGIN {FS=","} {print NF}'`
  F_log "_verify_count=$_verify_count"
  if [ "$_verify_count" = "0" ]; then
	# pass verify
	return 0
  fi

  _verify_language=`$INICONFIG_TOOL -r -f "$IWSS_HOME/intscan.ini" -s "Common" -o "lang"`

  k=0
  while [ $k -lt $_verify_count ]
  do
	k=`expr $k + 1`
	_verify_item=`echo $MIGRATE_FROM | cut -d ',' -f$k`
	_verify_product=`echo $_verify_item | cut -d '|' -f1`
	_verify_version=`echo $_verify_item | cut -d '|' -f2`
	_verify_platform=`echo $_verify_item | cut -d '|' -f3`

	F_log "_verify_product=$_verify_product"
	F_log "_verify_version=$_verify_version"
	F_log "_verify_language=$_verify_language"
	F_log "_verify_platform=$_verify_platform"

	if [ -z "$_verify_product" ] || [ -z "$_verify_version" ] || [ -z "$_verify_language" ]; then
	  # verification string format error
	  return 100
	fi
	if [ -z "$_verify_platform" ]; then
	  if [ "$_verify_product" = "$_product" ] && [ "$_verify_version" = "$_version" ]; then
		# ignore language check
		# pass verify
		return 0
	  fi
	else
	  if [ "$_verify_product" = "$_product" ] && [ "$_verify_version" = "$_version" ] &&
		 [ "$_verify_platform" = "$_platform" ]; then
		# ignore language check
		# pass verify
		return 0
	  fi
	fi
  done

  _cur_product=`$IWSS_HOME/iwssd -v | cut -d' ' -f1`
  _cur_version=`$IWSS_HOME/iwssd -v | cut -d' ' -f2 | cut -d'_' -f1`
  _cur_platform=`$IWSS_HOME/iwssd -v | cut -d' ' -f2 | cut -d'_' -f3`
  _cur_language=`$INICONFIG_TOOL -r -f "$IWSS_HOME/intscan.ini" -s "Common" -o "lang"`

  if [ -z "$_cur_platform" ]; then
	if [ "$_cur_product" = "$_product" ] && [ "$_cur_version" = "$_version" ]; then
	  # ignore language check
	  # pass verify
	  return 1
	fi
  else
	if [ "$_cur_product" = "$_product" ] && [ "$_cur_version" = "$_version" ] &&
	   [ "$_cur_platform" = "$_platform" ]; then
	  # ignore language check
	  # pass verify
	  return 1
	fi
  fi

  F_log "F_verify() out"
  return 100
}

F_PackageCheck()
{
	F_log "IN F_PackageCheck"

	_package=$1
	_extended=`expr "$_package" : '.*\(\..*\)'`

	mkdir -p $MIG_TMP_PATH
	if [ -f "$PRODUCT_INFO_FILE" ] ; then
		rm -f $PRODUCT_INFO_FILE
	fi

	if [ "$_extended" = ".zip" ] || [ "$_extended" = ".ZIP" ] ; then
		productInfo=`unzip -l $_package | grep "ProductInfo.xml"`
		config=`unzip -l $_package | grep "Configurations"`
		unzip -P trend $_package ProductInfo.xml -d $MIG_TMP_PATH
	elif [ "$_extended" = ".gz" ] || [ "$_extended" = ".GZ" ] ; then
		productInfo=`tar ztf $_package | grep "ProductInfo.xml"`
		config=`tar ztf $_package | grep "Configurations"`
		tar zxf $_package -C $MIG_TMP_PATH ProductInfo.xml
	elif [ "$_extended" = ".tar" ] || [ "$_extended" = ".TAR" ] ; then
		productInfo=`tar tf $_package | grep "ProductInfo.xml"`
		config=`tar tf $_package | grep "Configurations"`
		tar xf $_package -C $MIG_TMP_PATH ProductInfo.xml
	else
		F_log "Package format does not supported"
		return $ERR_INVALID_IMPORT_PKG
	fi

	converted=0

	if [ -z "$productInfo" ] && [ -z "$config" ] ; then
		F_log "Not compatitable package, should be convert first"
		mkdir -p $CDT_CONVERTED_PATH
		rm -f $CDT_CONVERTED_PACKAGE
		convert_result=`$CDT_PKG_COVERT_TOOL $_package $CDT_CONVERTED_PACKAGE`
		retCode=$?
		if [ "$retCode" -eq 255 ] ; then
			F_log "Bad CDT package"
			return $ERR_INVALID_IMPORT_PKG
		elif [ "$retCode" -eq 254 ] ; then
			F_log "Bad version package"
			return  $ERR_INVALID_VERSION
		else
			F_log "CDT package has been converted"
			converted=1
			productInfo=`tar tf $CDT_CONVERTED_PACKAGE | grep "ProductInfo.xml"`
			config=`tar tf $CDT_CONVERTED_PACKAGE | grep "Configurations"`
			tar xf $CDT_CONVERTED_PACKAGE -C $MIG_TMP_PATH ProductInfo.xml
			import_package=$CDT_CONVERTED_PACKAGE
		fi
	fi

	if [ -z "$productInfo" ] && [ -z "$config" ] ; then
		F_log "Corrupt package missing ProductInfo.xml and Configurations"
		return $ERR_INVALID_IMPORT_PKG
	elif [ ! -z "$productInfo" ] && [ -z "$config" ] ; then
		F_log "Corrupt package, missing ProductInfo.xml"
		return $ERR_INVALID_IMPORT_PKG
	elif [ -z "$productInfo" ] && [ ! -z "$config" ] ; then
		F_log "Corrupt package, missing Configurations"
		return $ERR_INVALID_IMPORT_PKG
	fi

	if [ ! -f $PRODUCT_INFO_FILE ] ; then
		F_log "Bad package"
		return $ERR_INVALID_IMPORT_PKG
	fi

	productName=`awk -F'"' '/<[Pp]roduct/ {print $2}' $PRODUCT_INFO_FILE`
	productVersion=`awk -F'"' '/<[Vv]ersion value/ {print $2}' $PRODUCT_INFO_FILE`
	productLanguage=`awk -F'"' '/<[Ll]anguage value/ {print $2}' $PRODUCT_INFO_FILE`
	productPlatform=`awk -F'"' '/<[Pp]latform value/ {print $2}' $PRODUCT_INFO_FILE`
	productHAType=`awk -F'"' '/<haType value/ {print $2}' $PRODUCT_INFO_FILE`
	productBuildNumber=`awk -F'"' '/<buildNumber value/ {print $2}' $PRODUCT_INFO_FILE`
	osVersion=`awk -F'"' '/<osVersion value/ {print $2}' $PRODUCT_INFO_FILE`

	F_log "$productName $productVersion $productLanguage $productPlatform $productHAType $productBuildNumber $osVersion"

	F_verify $productName $productVersion $productLanguage $productPlatform

	case $? in
		100)
			F_log "Package not support."
			return $ERR_INVALID_IMPORT_PKG
			;;
		0)
			F_log "Package verify successfully."
			return 0
			;;
		1)
			# For import, check HA configuration
			cur_build=`/etc/iscan/iwssd -v | awk -F'[_ ]' '{print $5}'`
			cur_os_ver=`cat /etc/ver.os`
			if [ "$import_type" == "--SYNC" ];then
				if [ "$productBuildNumber" != $cur_build ];then
					F_log "Import Package Build Number = $productBuildNumber"
					F_log "Current IWSVA Build Number  = $cur_build"
					F_log "For SYNC import, the IWSVA build number must be same."
					return $ERR_INVALID_IMPORT_PKG
				fi
				if [ "$osVersion" != "$cur_os_ver" ];then
					F_log "Import Package OS version = $osVersion"
					F_log "Current IWSVA OS version  = $cur_os_ver"
					F_log "For SYNC import, the IWSVA OS version must be same."
					return $ERR_INVALID_IMPORT_PKG
				fi
			fi

			if [ "$productHAType" == "single" -a `/usr/iwss/haagntcmd GetRole` == "$productHAType" ];then
				F_log "Need to ask user to choose fully or partially migration"
				return 1
			else
				F_log "Package verify successfully."
				return 0
			fi
	esac
}


###
F_lock()
{
  _current_pid=$$
  echo $_current_pid > $IWSS_HOME/$MODULE_NAME.pid
}

F_unlock()
{
	rm $IWSS_HOME/$MODULE_NAME.pid
}

hpIsProcessAlive()
{
	if [ -x /sbin/pidof ] ; then
		/sbin/pidof `basename $*` > /dev/null
	elif [ -x /bin/pidof ] ; then
		/bin/pidof `basename $*` > /dev/null
	else
		ps -ef | grep $* | grep -v grep > /dev/null
	fi
}

F_enable_content_cache()
{
	if [ -f ${CACHE_HELPER} ]; then
		_cache_status=`${INICONFIG_TOOL} -r -f ${CACHE_CFG} -s "Cache" -o "enable"`
		F_log "Cahce enable=$_cache_status"
		if [ "$_cache_status" = "yes" ]; then
			F_log "Prepare to enable content cache... ..."
			${PROMOTE_PERMISSION} "${CACHE_HELPER} EnableCache > /dev/null"
		fi
	fi
}

F_export()
{
	F_log "Begin to export ... ..."

	if [ -e $MIG_TMP_PATH ] ; then
		F_log "Delete $MIG_TMP_PATH"
		$PROMOTE_PERMISSION "rm -rf $MIG_TMP_PATH"
	fi

	mkdir -p $MIG_EXPORT_PATH
	mkdir -p $MIG_TMP_PATH/Configurations/
	$MIG_MAIN_TOOL -E $MIG_TMP_PATH "$mig_content" "$import_type" "$psql_parameter"
	result=$?
	if [ $result -eq 0 ] ; then
		F_UI_log "$MIG_RES_FILE" "export" "error10" "Error"
		return 154
	elif [ $result -eq 2 ] ; then
		F_UI_log "$MIG_RES_FILE" "export" "error11" "Error"
		return 155
	fi

	if [ -e $PRODUCT_INFO_FILE ]; then
		ex_haType=`/usr/iwss/haagntcmd GetRole`
		ex_build=`/etc/iscan/iwssd -v | awk -F'[_ ]' '{print $5}'`
		os_version=`cat /etc/ver.os`
		sed -i "/language/a\    <haType value=\"$ex_haType\" />\n    <buildNumber value=\"$ex_build\" />\n    <osVersion value=\"$os_version\" />" $PRODUCT_INFO_FILE
	fi

	F_log "Pack exported configurations."
	$PROMOTE_PERMISSION "tar -C $MIG_TMP_PATH -cf $1 Configurations ProductInfo.xml"
	$PROMOTE_PERMISSION "rm -rf $MIG_TMP_PATH"
	F_log "Configuration export finished."
	F_UI_log "$MIG_RES_FILE" "export" "message10" "Finish" "|$1"
}

F_set_timezone()
{
	_zone_str=`cat /etc/sysconfig/clock | grep "ZONE=" | cut -d'"' -f2`
	if [ -n "$_zone_str" ] ; then
		_zone_front=`echo $_zone_str | cut -d'/' -f1`
		_zone_rear=`echo $_zone_str | awk -F'/' '{if($NF=="Center"){NF-=1};print $NF}'`

		F_log "_zone_front=$_zone_front"
		F_log "_zone_rear=$_zone_rear"

		$PROMOTE_PERMISSION "/bin/cp --remove-destination /usr/share/zoneinfo/${_zone_str} /etc/localtime" >> $DEBUG_LOG_FILE
		$PROMOTE_PERMISSION "/usr/bin/rwconfig -ct -d '=' -f /etc/sysconfig/clock -s ZONE -n \"${_zone_str}\\\"\"" >> $DEBUG_LOG_FILE
		$PROMOTE_PERMISSION "/sbin/hwclock --systohc" >> $DEBUG_LOG_FILE
		F_log "Change timezone to $_zone_front/$_zone_rear"
		return 0
	fi
	return 1
}

F_set_hostname()
{
	_host_name=`cat /etc/sysconfig/network | grep "HOSTNAME=" | cut -d'=' -f2`
	if [ -n "$_host_name" ] ; then
		$PROMOTE_PERMISSION "hostname $_host_name" >> $DEBUG_LOG_FILE
	fi
}

F_set_crontab()
{
	$PROMOTE_PERMISSION "/etc/init.d/crond restart" >> $DEBUG_LOG_FILE
}

F_make_home_dir_for_OS_account()
{
	_pass_file="/etc/passwd"
	_tmp_pass="/tmp/tmp-pass"
	rm -f $_tmp_pass

	$PROMOTE_PERMISSION "cp $_pass_file $_tmp_pass"
	$PROMOTE_PERMISSION "chown iscan:iscan $_tmp_pass"
	_user_id_list=`cat $_tmp_pass | cut -d':' -f3`

	_count=0
	for _id in $_user_id_list
	do
		_count=`expr $_count + 1`

		if [ $_id -ge 500 ]; then
			F_log "_id=$_id, _count=$_count"
			_account_info=`sed -n "$_count,$_count p" $_tmp_pass`
			_user_info=`echo $_account_info | cut -d':' -f3`
			_group_info=`echo $_account_info | cut -d':' -f4`
			_homedir_info=`echo $_account_info | cut -d':' -f6`
			if [ ! -d "$_homedir_info" ]; then
				$PROMOTE_PERMISSION "mkdir -p $_homedir_info"
				F_log "mkdir -p $_homedir_info"
			fi
			$PROMOTE_PERMISSION "chown -R $_user_info:$_group_info $_homedir_info"
			$PROMOTE_PERMISSION "chmod 700 $_homedir_info"
		fi
	done
	rm -f $_tmp_pass
}


F_apply_system_config()
{
	F_log "Make the system configuration take effective... ..."

	F_make_home_dir_for_OS_account
	F_set_hostname
	F_set_timezone
	F_set_crontab

	if [ -f "/etc/resolv.conf.predhclient" ]; then
		$PROMOTE_PERMISSION "cp -f /etc/resolv.conf /etc/resolv.conf.predhclient"
	fi

	$PROMOTE_PERMISSION "/etc/init.d/network restart"  >> $DEBUG_LOG_FILE
	$PROMOTE_PERMISSION "/etc/init.d/iptables restart"  >> $DEBUG_LOG_FILE
}

F_import()
{
	TARGET=$1

	if [ "$restart_service" = "yes" -a "$mig_content" != "--DBPolicy" ]; then
		if [ "$ccr_import" = "no" ]; then
			F_log "Stop IWSVA daemon, then start database"
			$PROMOTE_PERMISSION "$IWSS_HOME/rcIwss stop --SYNC >> $DEBUG_LOG_FILE"
			$PROMOTE_PERMISSION "$IWSS_HOME/S99ISdatabase start >> $DEBUG_LOG_FILE"
		else
			$PROMOTE_PERMISSION "$IWSS_HOME/S99ISsvcmonitor stop >> $DEBUG_LOG_FILE"			
			F_log "CCR import reload+ begin"
		fi
	fi

	F_log "Unpack configurations... ..."
	if [ -e $MIG_TMP_PATH ] ; then
		$PROMOTE_PERMISSION "rm -rf $MIG_TMP_PATH"
	fi

	mkdir -p $MIG_TMP_PATH
	tar -C $MIG_TMP_PATH -xf $TARGET
	F_log "tmppath=$MIG_TMP_PATH, target=$TARGET"

	#Xin added to get the lang for InspectionFilter.xml
	#env_lang=`$INICONFIG_TOOL -r -f "$IWSS_HOME/intscan.ini" -s "Common" -o "lang"`
	#F_log "env_language=$env_lang"

	#if [ $env_lang != "en" ]; then
	#	tar xf ${MIG_PKG_HOME}/IWSVA_6.5-SP2_Linux.tar -C ${MIG_TMP_PATH} Configurations/InspectionFilter.xml
	#fi

	F_log "Begin to import configurations... ..."
	export MIG_IMPORT_TYPE="$import_type" # for migrate_network use.

	F_log "$MIG_MAIN_TOOL -I $MIG_TMP_PATH $mig_content $psql_parameter $import_type"
	$MIG_MAIN_TOOL -I $MIG_TMP_PATH "$mig_content" "$psql_parameter" "$import_type"
	result=$?
	if [ $result -eq 0 ] ; then
		F_UI_log "$MIG_RES_FILE" "import" "message01" "ImportFail"
	elif [ $result -eq 2 ] ; then
		F_UI_log "$MIG_RES_FILE" "import" "message01" "Error"
	fi

	#Xin recover the Inspection xml file after migration finished
	#if [ $env_lang != "en" ]; then
	#	cp -f ${MIG_TMP_PATH}/Configurations/InspectionFilter.xml /etc/iscan
	#fi

	F_log "sync certificates start"
	# Sync certificates from jscan to HTTPs
	# Sync certificates to https scan dir
	CERT_STORE="/etc/iscan/https/certstore/"
	rm -f ${CERT_STORE}/certificates/*.pem
	rm -f ${CERT_STORE}/certificates/rehash/*.[0-9]*
	rm -f ${CERT_STORE}/sync/active/*
	rm -f ${CERT_STORE}/sync/inactive/*
	echo -n > ${CERT_STORE}/default.trust
	echo -n > ${CERT_STORE}/default.untrust

	/usr/iwss/certsynctool/certsync_tohttpsdir.sh > /dev/null 2>&1

	/usr/iwss/CA_certs_transhash.sh \
		"${CERT_STORE}/sync/active/" \
		"${CERT_STORE}/certificates/" \
		"${CERT_STORE}/default.trust"

	/usr/iwss/CA_certs_transhash.sh \
		"${CERT_STORE}/sync/inactive/" \
		"${CERT_STORE}/certificates/" \
		"${CERT_STORE}/default.untrust"

	F_log "sync certificates finish"

	F_log "sync crl start"
	/var/iwss/https/certstore/script/rehash_crl.sh
	F_log "sync crl finish"
	
	F_log "Configuration import finished."
	F_UI_log "$MIG_RES_FILE" "import" "message03" "Finish"

	ldap_sync=`$INICONFIG_TOOL -r -f "$IWSS_HOME/intscan.ini" -s "user-identification" -o "user_identification_solution"`
	if [ "$ldap_sync" = "3" ]; then
		F_log "ldap sync"
		$LDAP_SYNC_TOOL
	fi

	if [ "$restart_service" = "yes" -a "$mig_content" != "--DBPolicy" ]; then
		if [ "$import_type" = "--FULL" ]; then
			F_apply_system_config
			F_log "after F_apply_system_config"
			F_UI_polling_start
			F_log "after F_UI_polling_start"
			/etc/iscan/AdminUI/uihelper reboot
			F_log "after reboot"
		elif [ "$ccr_import" = "no" ]; then

			# Apply the system setting
			F_apply_system_config

			F_log "Start IWSVA services, after import... ..."
			$PROMOTE_PERMISSION "$IWSS_HOME/S99ISdatabase stop >> $DEBUG_LOG_FILE"
			$PROMOTE_PERMISSION "$IWSS_HOME/rcIwss start --SYNC >> $DEBUG_LOG_FILE"
		else
			F_log "CCR import reload+ end"
			$PROMOTE_PERMISSION "$IWSS_HOME/S99ISproxy reload+ >> $DEBUG_LOG_FILE"
			$PROMOTE_PERMISSION "$IWSS_HOME/S99ISftp reload+ >> $DEBUG_LOG_FILE"
		fi
	elif [ "$restart_service" = "yes" -a "$mig_content" == "--DBPolicy" ]; then
		$PROMOTE_PERMISSION "$IWSS_HOME/S99ISproxy reload+ >> $DEBUG_LOG_FILE"
		$PROMOTE_PERMISSION "$IWSS_HOME/S99ISftp reload+ >> $DEBUG_LOG_FILE"
	fi

	F_enable_content_cache
	F_UI_not_import_partially_list
	F_UI_polling_start
}

# Check import package type
F_CheckImportPackage()
{
	local   check_result=$PKG_TYPE_NOTSUPPORT
	local   temp_import_package=$1

	F_PackageCheck "$temp_import_package"

	check_result=$?
	case $check_result in
		0)
			if [ "$converted" -eq 1 ] ; then
				mig_package=$CDT_CONVERTED_PACKAGE
				F_log "IWSS/SA package, has been converted [$mig_package]"
			else
				F_log "IWSVA $productVersion package"
			fi
			;;

		1)
			MIG_UPGRADE_VA="false"
			F_log "IWSVA $productVersion package"
			;;

		*)
			F_log "Package verify failed."
			check_result=$PKG_TYPE_NOTSUPPORT
			;;
	esac

	return $check_result
}

# Check if there is multiple instances running
F_CheckMultipleRunning()
{
	local check_result=`F_IsMultipleProcess`

	if [ $? -ne 0 ] ; then
		F_log "F_IsMultipleProcess() function error"
		return 100
	fi

	if [ "$check_result" = "true" ] ; then
		F_log "Another Import/Export is running"
		return 100
	fi

	return 0
}

# Remove the previous execute environment
F_init_running_env()
{
	$PROMOTE_PERMISSION "rm -f /tmp/import_polling_progress_start"
	$PROMOTE_PERMISSION "rm -f $DEBUG_LOG_FILE"
	#$PROMOTE_PERMISSION "rm -f $IMPORT_UI_LOG"
	#$PROMOTE_PERMISSION "rm -f $EXPORT_UI_LOG"

	$PROMOTE_PERMISSION "rm -rf /usr/iwss/migration/tmp"
}

# Check if DB can be connected
F_check_db_connection()
{
	if [ -z "$policy_db_host" -o -z "$policy_db_port" -o -z "$policy_db_name" -o -z "$policy_db_user" ]; then
		F_log "Cannnot check db's connection because of missing connection parameters."
		return 100
	fi

	if [ -n "$policy_db_password" ]; then
		export PGPASSWORD="$policy_db_password"
	fi

	$PSQL_CLIENT -h $policy_db_host -p $policy_db_port -U $policy_db_user -d $policy_db_name -c "\d" >/dev/null
	if [ $? -ne 0 ]; then
		F_log "Test DB connection failure, please check DB connection paramter."
		return 100
	fi

	return 0
}

F_GenerateScheduleFilePath() {
	file_name=`date +%Y-%m-%d`
	echo ${MIG_SCHEDULE_EXPORT_PATH}"/"${file_name}"_Config.tar"
}

F_DoPostExport() {
	file_name=$1
	md5_value=`F_DoGenerateMd5Sum $file_name`
	md5_full_value=${md5_value}" "${file_name}
	md5_path=${MIG_SCHEDULE_EXPORT_PATH}"/md5Set"
	if [ ! -f "$md5_path" ]; then
		touch $md5_path
	fi
	grep -q "${md5_value}" $md5_path
	if [ $? -eq 0 ]; then
		rm -f $file_name
	else
		echo $md5_full_value >> $md5_path
	fi

	return 0
}

F_DoGenerateMd5Sum() {
	md5_temp_file=${MIG_SCHEDULE_EXPORT_PATH}"/md5Set_tmp"
	if [ ! -f ${md5_temp_file} ]; then
		touch $md5_temp_file
	fi
	file_name=$1
	tmp_folder=${file_name}"_tmp"
	if [ ! -d ${tmp_folder} ]; then
		mkdir -p $tmp_folder
	fi
	tar xf ${file_name} -C ${tmp_folder}
	F_DoGetFileMd5Sum ${tmp_folder} ${md5_temp_file}
	echo `md5sum ${md5_temp_file} | cut -d " " -f 1`
	rm -rf ${md5_temp_file}
	rm -rf ${tmp_folder}
}

F_DoGetFileMd5Sum() {
	local file_name=$1
	local md5_temp_file=$2
	if [ -d "${file_name}" ]; then
		ls ${file_name} | while read name
		do
			local item=${file_name}"/"${name}
			F_DoGetFileMd5Sum "${item}" "${md5_temp_file}"
		done
	else
		echo "$file_name" | grep -q -E ".lock|tb_report_item|tb_report_template"
		if [ 0 -ne $? ]; then
			echo $file_name | grep -q "aucfg.ini"
			if [ 0 -eq $? ]; then
				sed -i -e '/^\[local_pattern_filename\]/,/^\[.*\]/d' $file_name
			fi
			echo $file_name | grep -q "intscan.ini"
			if [ 0 -eq $? ]; then
				sed -i -e '/^\[Pattern-Update\]/,/^\[.*\]/d' $file_name
			fi
			echo $file_name | grep -q "shadow"
			if [ 0 -eq $? ]; then
				chmod u+r $file_name
			fi
			 md5sum "${file_name}" | cut -d " " -f 1 >> ${md5_temp_file}
		fi
	fi
}

F_DeleteScheduleItem() {
   file_path=$1
   file_name=`echo $file_path | cut -d "/" -f 6`
   md5_path=${MIG_SCHEDULE_EXPORT_PATH}"/md5Set"
   rm -f $file_path
   sed -i "/${file_name}/"d ${md5_path}

   return 0
}

F_do_pre_check() {
	full_file_name=$1
	if [ -f ${full_file_name} ]; then
		rm -f $full_file_name
		md5_path=${MIG_SCHEDULE_EXPORT_PATH}"/md5Set"
		file_name=`echo $full_file_name | cut -d "/" -f 6`
		sed -i "/${file_name}/"d ${md5_path}
	fi
}

F_ScheduleCleanBackup()
{
        isNumber=`echo $MIG_SCHEDULE_LATEST_BACKUP_NUM | sed 's/[0-9]//g'`
        backupNum=`ls $MIG_SCHEDULE_EXPORT_PATH | grep -v "md5" | wc -l`
        F_log "CleanBackup=$MIG_SCHEDULE_CLEAN_BACKUP, LatestBackupNum=$MIG_SCHEDULE_LATEST_BACKUP_NUM"

        if [ "$MIG_SCHEDULE_CLEAN_BACKUP" != "yes" ] || [ -n "$isNumber" ] || [ ${MIG_SCHEDULE_LATEST_BACKUP_NUM} -ge $backupNum ]
        then
                F_log "No need clean up"
                return 0
        fi  

        deleteNum=$[${backupNum}-${MIG_SCHEDULE_LATEST_BACKUP_NUM}]
        backupFiles=`ls -rt $MIG_SCHEDULE_EXPORT_PATH | grep -v "md5" | head -n $deleteNum`
        backupArray=("$backupFiles")
        for file in $backupArray
        do 
                filePath=${MIG_SCHEDULE_EXPORT_PATH}/$file
                [ -f $filePath ] && F_DeleteScheduleItem $filePath
                F_log "delete backup file: $filePath"
        done
        return 0
}

###################### Main Migration Script ########################

F_init_running_env

F_get_default_values

# Check if execute it using 'iscan' account
F_check_user "iscan" "iscan"
if [ $? -ne 0 ]; then
	F_log "Invalid account, please execute this tool in 'iscan' account."
	exit $ERR_ACCOUNT_INVALID
fi

if [ "$1" = "-m" ]; then
	F_CheckMultipleRunning
	exit $?
fi

# Check number of parameters passed into
if [ "$#" -lt 1 ]; then
	F_usage
	F_log "Invalid parameters, please specify correct parameters to running this tools."
	exit $ERR_PARAMETER_INVALID
fi

# Check if multiple instance is running
F_CheckMultipleRunning
if [ $? -ne 0 ]; then
	F_log "Multiple instances is running, please retry after previous instance finished."
	exit $ERR_MUTIPLE_INSTANCE_RUNNING
fi

F_lock
F_log "Start to running migration... ..."

# Initialize and construct paramters
import_type="--PARTIAL"
restart_service="yes"
mig_action=""
mig_content=""
mig_package="$2"
psql_parameter=""
ccr_import="no"

for parameter in $@ ; do

	# migration action (check, export or import)
	echo "$parameter" | grep -q -e "^-C$"
	[ "$?" -eq 0 ] && mig_action="--check"

	echo "$parameter" | grep -q -e "^-I$"
	[ "$?" -eq 0 ] && mig_action="--import"

	#echo "$parameter" | grep -q -e "^-E$"
	#[ "$?" -eq 0 ] && mig_action="--export"
	if [ "$parameter" = "-E" ]; then
		mig_action="--export"
	fi

	echo "$parameter" | grep -q -e "^-S$"
	if [ "$?" -eq 0 ]; then
		mig_action="--schedule"
		mig_package=""
	fi

	echo "$parameter" | grep -q -e "^-D$"
	if [ "$?" -eq 0 ]; then
		mig_action="--delete"
	fi

	# import type (full, sync, partial)
	echo "$parameter" | grep -q -e "^--FULL$"
	[ "$?" -eq 0 ] && import_type="--FULL"

	echo "$parameter" | grep -q -e "^--SYNC$"
	[ "$?" -eq 0 ] && import_type="--SYNC"

	# service restart?
	echo "$parameter" | grep -q -e "^--NotApply$"
	[ "$?" -eq 0 ] && restart_service="no"

	# migration content (policy or cofnigure file)
	echo "$parameter" | grep -q -e "^--DBPolicy$"
	[ "$?" -eq 0 ] && mig_content="--DBPolicy"

	echo "$parameter" | grep -q -e "^--ConfigFile$"
	[ "$?" -eq 0 ] && mig_content="--ConfigFile"

	# PSQL connection parameter
	echo "$parameter" | grep -q -e "^--PSQLParameter=.*"
	[ "$?" -eq 0 ] && psql_parameter="$parameter"

	# CCR parameter
	echo "$parameter" | grep -q -e "^--CCR$"
	[ "$?" -eq 0 ] && ccr_import="yes"
done

# Construct DB connection parameter
if [ -z "$psql_parameter" ]; then
	policy_db_host=`$INICONFIG_TOOL -r -f $IWSS_INTSCAN_INI -s Database -o DB_HOSTNAME`
	policy_db_port=`$INICONFIG_TOOL -r -f $IWSS_INTSCAN_INI -s Database -o DB_PORT`
	policy_db_name=`$INICONFIG_TOOL -r -f $IWSS_INTSCAN_INI -s Database -o DB`
	policy_db_user=`$INICONFIG_TOOL -r -f $IWSS_INTSCAN_INI -s Database -o User`
	policy_db_password=""
else
	temp_parameter=`echo $psql_parameter | cut -d'=' -f2`
	policy_db_host=`echo $temp_parameter | cut -d':' -f1`
	policy_db_port=`echo $temp_parameter | cut -d':' -f2`
	policy_db_name=`echo $temp_parameter | cut -d':' -f3`
	policy_db_user=`echo $temp_parameter | cut -d':' -f4`
	policy_db_password=`echo $temp_parameter | cut -d':' -f5-`
fi

F_log "mig_action=$mig_action, import_type=$import_type, mig_content=$mig_content, mig_package=$mig_package"
F_log "psql_parameter=$psql_parameter"

# check if patch is running when import
if [ "$mig_action" == "--check" -o "$mig_action" == "--import" ]; then
	PATCH_PID_FILE="/etc/iscan/pids/patch.pid"
	unset patch_pid
	[ -f "$PATCH_PID_FILE" ] && patch_pid=`cat $PATCH_PID_FILE`
	if [ -n "$patch_pid" ]; then
		process=`ps aux|grep "$patch_pid"|grep -v -E '(grep)|(\<defunct\>)'|grep -E '(PatchExe.sh)|(RollbackExe.sh)'`
		if [ -n "$process" ]; then
			F_log "The system is installing a patch, please trigger after installation."
			F_unlock
			exit $ERR_PATCH_RUNNING
		fi
	fi
fi

# Execute the real action
case "$mig_action" in

	"--check")
		F_CheckImportPackage "$mig_package"
		RET=$?
		;;

	"--export" | "--schedule")
		if [ "$mig_action" == "--schedule" ]; then
			if [ ! -d "$MIG_SCHEDULE_EXPORT_PATH" ]; then
				mkdir -p $MIG_SCHEDULE_EXPORT_PATH
			fi
			mig_package=`F_GenerateScheduleFilePath`
			F_do_pre_check $mig_package
			echo $mig_package
		fi
		F_check_db_connection
		if [ $? -ne 0 ]; then
			RET=$ERR_DB_CONNECT_FAILURE
		else
			MODULE="Export"
			$PROMOTE_PERMISSION "rm -f $EXPORT_UI_LOG"
			F_export $mig_package
			RET=$?
			if [ $? -eq 0 ]; then
				if [ "$mig_action" == "--schedule" ]; then
					F_DoPostExport "$mig_package"
					F_ScheduleCleanBackup
					RET=$?
				fi
			fi
		fi
		;;

	"--delete")
		F_DeleteScheduleItem "$mig_package"
		RET=$?
		;;

	"--import")
		 F_check_db_connection
		 if [ $? -ne 0 ]; then
			RET=$ERR_DB_CONNECT_FAILURE
		 else
			F_CheckImportPackage "$mig_package"
			RET=$?
			if [ "$RET" -ne "$PKG_TYPE_NOTSUPPORT" ] && [ "$RET" -ne "$ERR_INVALID_VERSION" ] ; then
				F_log "Begine call F_import()... ..."
				MODULE="Import"
				$PROMOTE_PERMISSION "rm -f $IMPORT_UI_LOG"
				F_import $mig_package
				RET=$?
			fi
		fi
		;;
	*)
		F_log "Invalid action."
		F_usage
		RET=$ERR_PARAMETER_INVALID
		;;
esac

F_unlock

sleep 10
$PROMOTE_PERMISSION "$IWSS_HOME/S99ISsvcmonitor start >> $DEBUG_LOG_FILE"	
F_log "Exit migration.sh with $RET"

exit $RET
