#!/usr/bin/env python
# -*- coding: utf-8 -*-

import collections
from copy import deepcopy
import os
import zlib
import json
import logging
import logging.config
import glob
import sys
import os.path
sys.path += glob.glob(os.path.dirname(__file__) + '/schema/')
import uuid
from threading import Thread
from common_id import logformat
try:
    from statDef import reportStats
    from statDef import dashboardStats
except Exception, e:
    reportStats = {}
    dashboardStats = {}
try:
    from statsDef import valueCalMethod
except Exception, e:
    valueCalMethod = {
        'logTimes':0,
        'specField':1,
        'noValue' :-1
    }


FILE_NOT_EXIST = -1
OK = 0
SCHEMA_ERROR = -2
TOTAL_TRAFFIC = 'CLF_Total'
RBAC = 'CLF_UserName'
ISCALCRBAC = False

class browsingTimeThread(Thread):
    def __init__( self, **kwds):
        Thread.__init__( self, **kwds )
        self.__setDaemon(True) # set thread into daemon thread
    def run(self):
        pass
    def __setDaemon(self,isDaemon):
        Thread.setDaemon(self,isDaemon)


class stats(object):

    def __init__(self, upload_stats_folder, repStatsDef=None, dashStatsDef=None, reports=[]):
        if repStatsDef:
            reportStats_c = deepcopy(repStatsDef)
        else:
            reportStats_c = deepcopy(reportStats)
        if dashStatsDef:
            dashboardStats_c = deepcopy(dashStatsDef)
        else:
            dashboardStats_c = deepcopy(dashboardStats)
        #browsTimeStats base columnID , for example browsTimeStats base ID is "CLF_UserName"
        browsingTimeDef = reportStats_c.get('browsing_time')
        if browsingTimeDef:
            self.browsingStatsBaseIndex = browsingTimeDef['logFields'].index('CLF_UserName')
            self.browseTimeBaseStats = 'browsing_time'
        else:
            self.browsingStatsBaseIndex = -1
            self.browseTimeBaseStats = None
            self.browseTimeCountIndex = -1

        self.statsUserIndex = collections.defaultdict()   # for RBAC 'CLF_UserName' index
        self.stats_index = self.__parser_stats_schema(reportStats_c,logformat)
        #logging.info(dashboardStats_c)
        self.reports = reports
        self.dashboard_index = self.__parser_dashboard_schema(dashboardStats_c,logformat)
        self.save_stats_folder = upload_stats_folder
        if upload_stats_folder:
            self.tmp_root = os.path.dirname(__file__) + '/tmpfs'
            if not os.path.exists(self.tmp_root):
                os.makedirs(self.tmp_root)

    def setSchema(self, repStatsDef, dashStatsDef, reports=[]):
        reportStats_c = deepcopy(repStatsDef)
        dashboardStats_c = deepcopy(dashStatsDef)
        browsingTimeDef = reportStats_c.get('browsing_time')
        if browsingTimeDef:
            self.browsingStatsBaseIndex = browsingTimeDef['logFields'].index('CLF_UserName')
            self.browseTimeBaseStats = 'browsing_time'
        else:
            self.browsingStatsBaseIndex = -1
            self.browseTimeBaseStats = None
            self.browseTimeCountIndex = -1
        self.stats_index = self.__parser_stats_schema(reportStats_c,logformat)
        self.dashboard_index = self.__parser_dashboard_schema(dashboardStats_c,logformat)
        self.reports = reports


    def __parser_stats_schema(self, reportStats, logformat):
        assert isinstance(reportStats, dict)
        assert isinstance(logformat,list)
        format_index = collections.defaultdict()
        for item_index,item in enumerate(logformat):
            format_index[item]=item_index
        Stats_index = deepcopy(reportStats)
        for statName, value in Stats_index.iteritems():
            logFields = value['logFields']
            if 'CLF_UserName' in logFields:
                self.statsUserIndex[statName] = logFields.index('CLF_UserName')
            else:
                self.statsUserIndex[statName] = None
            for indx,groupID in enumerate(logFields):
                logFields[indx] = format_index[groupID]


            if  isinstance(value['valueCal'], list):
                for i, Cal in enumerate(value['valueCal']):
                    if Cal > 0:
                        if isinstance(value['valueField'][i], list):
                            field_index = []
                            for f in value['valueField'][i]:
                                field_index.append(format_index[f])
                            value['valueField'][i] = field_index
                        else:
                            value['valueField'][i] = format_index[value['valueField'][i]]
                    elif Cal == -1: # if Cal == -1, calcu browseTimeStats
                        self.browsingStatsBaseIndex = reportStats[statName]['logFields'].\
                                        index('CLF_UserName')
                        self.browseTimeBaseStats = statName
                        self.browseTimeCountIndex = i
            else:
                if value['valueCal'] > 0:
                    if isinstance(value['valueField'], list):
                        field_index = []
                        for f in value['valueField']:
                            field_index.append(format_index[f])
                        value['valueField'] = field_index
                    else:
                        value['valueField'] = format_index[value['valueField']]
                elif value['valueCal'] == -1:
                    self.browsingStatsBaseIndex = reportStats[statName]['logFields'].\
                                        index('CLF_UserName')
                    self.browseTimeBaseStats = statName
                    self.browseTimeCountIndex = 0


            logFilters =  value['logFilters']
            logFilters_tmp  = collections.defaultdict()
            for key in logFilters.iterkeys():
                logFilters_tmp[format_index[key]] = logFilters[key]

            if(len(logFilters_tmp) > 0):
                logFilters.clear()
                value['logFilters'] = dict(logFilters_tmp)

            if not value.has_key('interval'):
                value['interval'] = 3600
        #print Stats_index
        self.stats_index = Stats_index
        return Stats_index


    def __parser_dashboard_schema(self, dashboardStats, format):
        assert isinstance(dashboardStats, dict)
        assert isinstance(format,list)
        format_index = {}

        for item_index,item in enumerate(format):
            format_index[item]=item_index

        dashboard_index = dict(dashboardStats)
        for value in dashboard_index.itervalues():
            logFields = value['logFields']
            if logFields[0] is not TOTAL_TRAFFIC:
                for indx,groupID in enumerate(logFields):
                    if groupID == 'CLF_Timestamp':
                        logFields[indx] = format_index['CLF_UTCTimeStamp']
                    else:
                        logFields[indx] = format_index[groupID]
                #groupID0 = format_index[logFields[0]]
                #logFields[0] = groupID0

            if value['valueCal'] > 0 and value['valueCal'] != 2 :
                if isinstance(value['valueField'], list):
                    field_index = []
                    for f in value['valueField']:
                        field_index.append(format_index[f])
                    value['valueField'] = field_index
                else:
                    value['valueField'] = format_index[value['valueField']]
            logFilters =  value['logFilters']
            logFilters_tmp  = {}
            for key in logFilters.iterkeys():
                logFilters_tmp[format_index[key]] = logFilters[key]

            if(len(logFilters_tmp) > 0):
                logFilters.clear()
                value['logFilters'] = dict(logFilters_tmp)
        #print dashboard_index
        return dashboard_index


    def set_upload_folder(self, upload_stats_folder):
        if not upload_stats_folder:
            self.save_stats_folder = None
        if not os.path.exists(upload_stats_folder):
            return -1
        self.save_stats_folder = upload_stats_folder

    def __log_filter(self, log_list, index, filter):
        ret = True

        if filter == 'notEmpty':
            if log_list[index] == '':
                ret = False
        else:
            #if log_list[index] == '':
            #    return False
            if isinstance(filter[0], tuple):
                for item in filter:
                    if log_list[index] == '':
                        ret = False
                        break
                    if len(item) == 1 and int(log_list[index]) < item[0]:
                        ret = False
                        break
                    elif len(item) == 2 and (int(log_list[index]) < item[0] or int(log_list[index]) > item[1]):
                        ret = False
                        break
            elif isinstance(filter[0], str) or isinstance(filter[0], unicode):
                if log_list[index] not in filter:
                    ret = False
        return ret


    def __calcBrowsingTimeStats(self, statsName, rawStats, countIndex = -1):
        if not statsName  and not isinstance(rawStats, dict):
            logging.info('Browsing time stats error, statsName:%r,rawStats \
                         type:%r', statsName, type(rawStats))
            return  None
        if self.browsingStatsBaseIndex == -1:
            logging.info('browse time stats definition base ID error')
            return  None

        userCounter = collections.Counter()
        browseStatsCounter = collections.Counter()
        browsingTimeStast = []
        if self.browseTimeBaseStats == 'browsing_time':
            for itemKey, itemCount in  rawStats.iteritems():
                userCounter[itemKey[self.browsingStatsBaseIndex]] += itemCount

            for itemKey, itemCount in  rawStats.iteritems():
                baseID = itemKey[self.browsingStatsBaseIndex]
                browseTime = itemCount*60/userCounter[baseID]
                rawStats[itemKey] = browseTime
        else:
            for itemKey, itemCount in  rawStats.iteritems():
                userCounter[itemKey[self.browsingStatsBaseIndex]] += itemCount[countIndex]

            for itemKey, itemCount in  rawStats.iteritems():
                baseID = itemKey[self.browsingStatsBaseIndex]
                browseTime = itemCount[countIndex]*60/userCounter[baseID]
                itemCount[countIndex] = browseTime
        return browsingTimeStast

    def __storageStas(self,cid, timestamp, dashStats, reportStats, facetStats, browseStats):
        uuid_value = uuid.uuid1()
        if reportStats:
            stats_raw = json.dumps(reportStats)
            stats_compress = zlib.compress(stats_raw)
            #cid _uuid_ report_stats_timestamp
            filename = '%s_%s_report_stats_%r'%(cid,uuid_value,timestamp)

            tmp_file = os.path.join(self.tmp_root,filename)
            stats_file = open(tmp_file, 'wb+')
            stats_file.write(stats_compress)
            stats_file.close()
            cmd = ''.join('mv %s %s'%(tmp_file,self.save_stats_folder))
            os.system(cmd)

        if facetStats:
            stats_raw = json.dumps(facetStats)
            stats_compress = zlib.compress(stats_raw)
            filename = '%s_%s_facet_stats_%r'%(cid,uuid_value,timestamp)
            logging.debug('filename:%r', filename)
            tmp_file = os.path.join(self.tmp_root,filename)
            logging.debug('tmp file:%r', tmp_file)
            stats_file = open(tmp_file, 'wb+')
            stats_file.write(stats_compress)
            stats_file.close()
            cmd = ''.join('mv %s %s'%(tmp_file,self.save_stats_folder))
            os.system(cmd)

        if dashStats:
            stats_raw = json.dumps(dashStats)
            stats_compress = zlib.compress(stats_raw)
            filename = '%s_%s_dashbd_stats_%r'%(cid,uuid_value,timestamp)
            tmp_file = os.path.join(self.tmp_root,filename)
            stats_file = open(tmp_file, 'wb+')
            stats_file.write(stats_compress)
            stats_file.close()
            cmd = ''.join('mv %s %s'%(tmp_file,self.save_stats_folder))
            os.system(cmd)



    def calculate_stats(self, cid, timestamp, fileobj, data=None, ):
        list_stats = collections.defaultdict()
        dash_stats = []
        '''
        for stats every entry alloc Counter, for example: malware_location, userbyMal_location
        and so on
        '''
        Stats_index = self.stats_index
        dashboard_index = self.dashboard_index
        for key in Stats_index.iterkeys():
            stat = collections.Counter()
            if not list_stats.has_key(Stats_index[key]['interval']):
                list_stats[Stats_index[key]['interval']] = {}
            if isinstance(Stats_index[key]['valueCal'], list):
                list_stats[Stats_index[key]['interval']][key] = collections.defaultdict(list)
            else:
                list_stats[Stats_index[key]['interval']][key] = stat   # list_stats = {interval:{statsname:counter({})}},
                                                                   #interval:statsname:counter, collections.Counter()

        for key in dashboard_index.iterkeys():
            stat = collections.Counter()
            aTuple = (key,stat)
            dash_stats.append(aTuple)

        if not fileobj:
            logging.info('no need to open file to read stats, use data param')
        else:
            data = fileobj.read()

        lines = data.split('\n')
        for line in lines:
            if line == '':
                continue

            log_list = line.split('\t')
            try:
                for key, value in Stats_index.iteritems(): #value is also a dict
                    filter_set = True
                    if(len(value['logFilters']) > 0):
                         for k, filter in value['logFilters'].iteritems():
                             if not self.__log_filter(log_list, k, filter):
                                 filter_set = False
                                 break

                    if filter_set :
                        if  isinstance (value['valueCal'], list):
                            subStats = list_stats[value['interval']][key][tuple([log_list[index] for index in value['logFields']])]
                            if not subStats:
                                #if ISCALCRBAC:# for RBAC need  RBAC's column
                                list_stats[value['interval']][key][tuple([log_list[index] for index in value['logFields']])] = \
                                                                              [0]*(len(value['valueCal']))
                                subStats = list_stats[value['interval']][key][tuple([log_list[index] for index in value['logFields']])]

                            for i, Cal in enumerate(value['valueCal']):
                                if callable(Cal):
                                    param = []
                                    if isinstance(value['valueField'][i], list):
                                        param = [ int(log_list[x]) for x in value['valueField'][i]]
                                    else:
                                        param = [ int(log_list[value['valueField'][i]])]

                                    subStats[i] += apply(value['valueCal'][i], param)
                                elif Cal == valueCalMethod['specField']:
                                    if isinstance(value['valueField'][i], list):
                                        subStats[i] += int(log_list[value['valueField'][i][0]])
                                    else:
                                        subStats[i] += int(log_list[value['valueField'][i]])
                                else:
                                    subStats[i] += 1
                        else:
                            if callable(value['valueCal']):
                                param = []
                                if isinstance(value['valueField'], list):
                                    param = [ int(log_list[x]) for x in value['valueField'] ]
                                else:
                                    param = [ int(log_list[value['valueField']]) ]
                                list_stats[value['interval']][key][tuple([log_list[index] for index in value['logFields']])] \
                                                        += apply(value['valueCal'], param)
                            elif value['valueCal'] == 1: # list_stats[i][1] is counter
                                if int(log_list[value['valueField']]) <= 4000000000
                                    list_stats[value['interval']][key][tuple([log_list[index] for index in value['logFields']])] \
                                               += int(log_list[value['valueField']])
                            else:
                                list_stats[value['interval']][key][tuple([log_list[index] for index in value['logFields']])] \
                                        +=1
            except Exception,e:
                logging.exception('stats error:%r',e)
                logging.info('raw log is :%r',log_list)

            try:
                i = 0
                for key, value in dashboard_index.iteritems():
                    filter_set = True
                    if(len(value['logFilters']) > 0):
                         for k, filter in value['logFilters'].iteritems():
                             if not self.__log_filter(log_list, k, filter):
                                 filter_set = False
                                 break
                    if filter_set :
                         if callable(value['valueCal']):
                            param = []
                            if isinstance(value['valueField'], list):
                                param = [int(log_list[x]) for x in value['valueField'] ]
                            else:
                                param = [ int(log_list[value['valueField']]) ]
                            if len(value['logFields']) > 1:
                                if value['logFields'][0] == TOTAL_TRAFFIC:
                                    dash_stats[i][1][tuple([log_list[index] for index in value['logFields']])]\
                                             += apply(value['valueCal'], param)
                                else :
                                    dash_stats[i][1][tuple([log_list[index] for index in value['logFields']])] \
                                             += apply(value['valueCal'], param)
                            else:
                                if value['logFields'][0] == TOTAL_TRAFFIC:
                                    dash_stats[i][1][value['logFields'][0]] += apply(value['valueCal'], param)
                                else :
                                    dash_stats[i][1][log_list[value['logFields'][0]]] += apply(value['valueCal'], param)
                         elif value['valueCal'] == 1:
                            if len(value['logFields']) > 1:
                                if value['logFields'][0] == TOTAL_TRAFFIC:
                                    dash_stats[i][1][tuple([log_list[index] for index in value['logFields']])]\
                                                     += int(log_list[value['valueField']])
                                else :
                                    dash_stats[i][1][tuple([log_list[index] for index in value['logFields']])] \
                                             += int(log_list[value['valueField']])
                            else:
                                if value['logFields'][0] == TOTAL_TRAFFIC:
                                    dash_stats[i][1][value['logFields'][0]] += int(log_list[value['valueField']])
                                else :
                                    dash_stats[i][1][log_list[value['logFields'][0]]] += int(log_list[value['valueField']])
                         elif  value['valueCal'] == 2: # ccca dashboard
                             dash_stats[i][1][tuple([log_list[index] for index in value['logFields']])] +=1
                         else:
                            if len(value['logFields']) > 1:
                                if value['logFields'][0] == TOTAL_TRAFFIC:
                                    dash_stats[i][1][tuple([log_list[index] for index in value['logFields']])]\
                                             +=1
                                else :
                                    dash_stats[i][1][tuple([log_list[index] for index in value['logFields']])] \
                                             += 1

                            else:
                                if value['logFields'][0] == TOTAL_TRAFFIC:
                                    dash_stats[i][1][value['logFields'][0]] +=1
                                else:
                                    dash_stats[i][1][log_list[value['logFields'][0]]] +=1

                    i +=1
            except Exception,e:
                logging.exception('dashboard error:%r',e)
                logging.info('raw log is :%r',log_list)

        json_dict = {}
        json_dict_facet = {}
        json_dict_dash = {}
        json_dict_browing = {}
        for dur, stats in list_stats.iteritems():
            if dur == 3600:
                for name, counter in stats.iteritems():
                    if name == self.browseTimeBaseStats:
                        json_dict_browing = \
                             self.__calcBrowsingTimeStats(name, counter, self.browseTimeCountIndex)
                    if len(counter):
                        if type(counter) == type(collections.defaultdict(list)):
                            # calc RBAC
                            if ISCALCRBAC:
                                statsTmp = []
                                for statsCol, statsCount in counter.iteritems():
                                    # TODO calc RBAC
                                    rbacValue = 2**32-1
                                    statsTmp.append(list(statsCol)+[rbacValue] + statsCount)
                                json_dict_facet.update({name:statsTmp})
                            else:
                                if name in self.reports:
                                    json_dict.update({name:[list(k)+v for k, v in counter.iteritems()]})
                                else:
                                    json_dict_facet.update({name:[list(k)+v for k, v in counter.iteritems()]})
                        else:
                            if name in self.reports:
                                json_dict.update({name:[list(k)+[v] for k, v in counter.iteritems()]})
                            else:
                                json_dict_facet.update({name:[list(k)+[v] for k, v in counter.iteritems()]})

            elif dur == 300:
                pass

        for dashboard_item in dash_stats:
            name = dashboard_item[0]
            counter = dashboard_item[1]
            if len(counter):
                dash_list = []
                for k, v in counter.iteritems():
                    if isinstance(k, tuple) and len(k) < 3:
                        dash_list.append(list(k)+[v])
                    else:
                        dash_list.append([k]+[v])
                json_dict_dash.update({name:dash_list})

        if not self.save_stats_folder: # if upload folder is NULL, stats will return to caller.
            statsData = {'dash':json_dict_dash, 'stats':json_dict_facet}
            return statsData
        logging.debug('json_dict_dash:%r', json_dict_dash)
        logging.debug('json_dict_facet:%r', json_dict_facet)
        logging.debug('json_dict_report:%r', json_dict)
        self.__storageStas(cid, timestamp, json_dict_dash, json_dict, json_dict_facet, json_dict_browing)
        return OK


def main():
    import skynetStatDefRBAC
    ss = stats(None, skynetStatDefRBAC.skynetRepStats,skynetStatDefRBAC.skynetDashStats)
    fileObj = open('1348627412_trend', 'rb')
    ss.calculate_stats('trend',1348627412,fileObj)
    fileObj.close()



if __name__ == '__main__':
    main()

