#!/bin/sh

#Goto the end of file before you start to read this file.
#We assume the pwd has extracted files and dir

# Naming convention
# F_  means a subroutine

#.for local debug
#Env setup:
# mkdir -p /etc/iscan/patch/backup
# mkdir -p /etc/iscan/patch/config
# mkdir -p /etc/iscan/patch/log
# mkdir -p /usr/iwss/AdminUI/tomcat/patch/
# mkdir -p /usr/iwss/AdminUI/tomcat/webapps/ROOT/html
# cp patch_management.sample.ini /etc/iscan/patch/config/
# adduser iscan
#Execute && debug:
# cd ../build/; ./genpatch.sh; cp iwss_31_hotfix-6.1.1786.2.tgz /usr/iwss/AdminUI/tomcat/patch/
# cd xxxxx/src/script;
# ./PatchExe.sh untar /usr/iwss/AdminUI/tomcat/patch/iwss_31_hotfix-6.1.1786.2.tgz
# cat /etc/iscan/patch/log/patch.log
# ./PatchExe.sh apply
# cat /etc/iscan/patch/log/patch.log
#  ./RollbackExe.sh 6.1.1786.2
# cat /etc/iscan/patch/log/patch.log

#herb_todo
echo "p0=$0, p1=$1, p2=$2, p3=$3" >> /tmp/ttt

F_usage() {
    echo "Usage: $0 install_root_dir backup_dir (upload_check|apply_now) "
    echo "This script should backup for this patch, stop the daemon, apply this patch and start daemons."
}

CHECK_OR_APPLY="$1"
if [ -z "$CHECK_OR_APPLY" ] ; then
    echo "3nd argument should be either upload_check or apply_now"
    F_usage
    exit 4
fi

INSTALL_DIR="$2"
if [ -z "$INSTALL_DIR" ] ; then
    echo "1st argument should be installation directory."
    F_usage
    exit 2
fi

if [ "${CHECK_OR_APPLY}" = "apply_now" ] ; then
        BACKUP_DIR="$3"
        if [ -z "$BACKUP_DIR" ] ; then
            echo "2nd argument should be backup directory."
            F_usage
            exit 3
        fi
fi

APP_PATCH_ROOT=/etc/iscan
PATCHINFOINI=stargate_patch.ini
PATCHMGMINI=${INSTALL_DIR}/patch/config/patch_management.ini

. ./ini_util.sh
. ./stargate_patch_common.sh
. ./patch_action_extra.sh

PATCH_NUMBER=`F_ini_get $PATCHINFOINI PatchInfo Number`

#
# Update a patch file record
# param: patch_file_info_line
#
# return 0
#
F_add_to_patch_record(){

      _file_info="$1"
      _SECTION="$2"
      _num_files=`F_ini_get ${PATCHMGMINI} ${_SECTION} NumberOfAppliedFiles`

      if [ -z "$_num_files" ] ; then
          _num_files=0
      fi

      RANGE_START="/^\[${_SECTION}\]/"
      RANGE_END="/^\[.*\]/"

      MY_SEDCMD_1="${RANGE_START},${RANGE_END}p"
      _R=`sed -n -e "${MY_SEDCMD_1}" ${PATCHMGMINI} |grep ${_filepath} | wc -l`

     if  [ $_R = 0 ] ; then
         F_ini_add_key "$PATCHMGMINI" "$_SECTION" "File$_num_files" "$_file_info"
         _new_num_files=`expr $_num_files + 1`
         F_ini_add_key "$PATCHMGMINI" "$_SECTION" "NumberOfAppliedFiles" "$_new_num_files"
     elif  [ $_R = 1  ]; then
         _R=`sed -n -e "${MY_SEDCMD_1}" ${INI_F} |grep ${_filepath} | cut -d' ' -f1 | cut -d'=' -f1`
         F_ini_add_key "$PATCHMGMINI" "$_SECTION" "$_R" "$_file_info"
     else
         #printf "warning since dupliated entris %s in the section [%s] \n" "$_filepath" "$_SECTION"
         _R=`sed -n -e "${MY_SEDCMD_1}" ${INI_F} |grep ${_filepath} |tail -n 1 | cut -d' ' -f1 | cut -d'=' -f1`
         F_ini_add_key "$PATCHMGMINI" "$_SECTION" "$_R" "$_file_info"
     fi
    return 0
}

#
# detect if a newer patch file is on the system
# param: full path
# param: file version that is going to apply
#
# return: 0 if the system has a newer patch
#
F_detect_previous_patch_file(){

    _filepath="$1"
    _fileversion="$2"
    _num_files=`F_ini_get ${PATCHMGMINI} AppliedFileRecord NumberOfAppliedFiles`

    RANGE_START="/^\[${APPLIED_RECORD_SECTION}\]/"
    RANGE_END="/^\[.*\]/"

    MY_SEDCMD_1="${RANGE_START},${RANGE_END}p"
    _R=`sed -n -e "${MY_SEDCMD_1}" ${PATCHMGMINI} |grep ${_filepath} | wc -l`

     if  [ $_R = 0 ] ; then
         #printf "no such entry  [%s]\n" "$_filepath"
         return 1
     elif [ $_R -gt 1 ] ; then
         printf "Warning duplicated [%s]  in APPLIED_RECORD_SECTION\n" "$_filepath"
     fi

     _R=`sed -n -e "${MY_SEDCMD_1}" ${INI_F} |grep ${_filepath}|tail -n 1`
     _file_paramter_index=0
     for paramter in $_R
     do
         if [ "$_file_paramter_index" = "1" ] ; then
             _applied_file_version=$paramter
         fi
         _file_paramter_index=`expr $_file_paramter_index + 1 `
     done

     if [ $_applied_file_version -gt $_fileversion ] ; then
        #printf "current file v[%s] is older than the applied file v[%s]\n" "$_R" "$_fileversion"
        return 0
     fi
     
    return 1
}

# backup a list of files to a dir
# $1 install root dir
# $2 backup dir
F_backup(){

    echo "Backing up the files that are going to be overwritten."

    _index=0
    _install_root=$1
    _backup_dir=$2

    _backup_file_list=
    _smart_backup_file_list=
    _smart_backup_root=smart_backup_root/

    _number_of_file_items=`F_ini_get $PATCHINFOINI PatchInfo NumberOfFileItems`
    if [ $? -ne 0 ] ; then
        echo "Cannot get number of files in the patch ini."
        return 1
    fi

    _smart_backup=`F_ini_get $PATCHINFOINI PatchInfo SmartBackup`
    if [ $? -ne 0 ] ; then
        echo "Default patch ini SmartBackup is 1."
        _smart_backup=1
    fi

    #backup applied record into tmp #force delete previous tmp
    F_ini_delete_section "$PATCHMGMINI" "$TMP_SECTION" > /dev/null
    _orig_section=`F_ini_get_section    "$PATCHMGMINI" "$APPLIED_RECORD_SECTION"`
    if [ $? -ne 0 ] ; then
        echo "The $PATCHMGMINI $APPLIED_RECORD_SECTION is empty."
    fi

    F_ini_append_section "$PATCHMGMINI" "$TMP_SECTION" "$_orig_section"

    while [ "$_index" -lt $_number_of_file_items ]
    do
        _fileName=`P_get_file_info $PATCHINFOINI PatchInfo $_index name`
        _fullPath=`P_get_file_info $PATCHINFOINI PatchInfo $_index path`
        _thisVersion=`P_get_file_info $PATCHINFOINI PatchInfo $_index version`

        #if the system has newer version, we won't backup
        F_detect_previous_patch_file ${_fullPath} ${_thisVersion}

        if [ "$?" = "0" ] ; then
            if [ "$_smart_backup" = "1" ] ; then
                echo "Smart backup ${_fullPath}."
                _smart_backup_file=${_smart_backup_root}/${_fullPath}

                #cp the smart backup to a specific sub dir
                mkdir -p `dirname ${_smart_backup_file}`
                cp -a files/$_fileName $_smart_backup_file

                #set right mode and permission before going into smart backup
                _owner=`P_get_file_info $PATCHINFOINI PatchInfo $_index owner`
                if [ -n "$_owner" ] ; then
                    chown $_owner $_smart_backup_file
                fi
                _permission=`P_get_file_info $PATCHINFOINI PatchInfo $_index permission`
                if [ -n "$_permission" ] ; then
                    chmod $_permission $_smart_backup_file
                fi

                #delete the first '/' is it exist
                _my_first_char=`expr substr "$_fullPath" 1 1`
                if [ "$_my_first_char" = "/" ] ; then
                    _fullPath=`echo $_fullPath | sed -e 's/^.//'`
                fi
                _smart_backup_file_list="${_smart_backup_file_list} ${_fullPath}"
                echo "Add ${_fullPath} to smart backup"

                #add smart backup to the backup record
                #they will be rolling back into this version
                _f_info=`P_get_file_info $PATCHINFOINI PatchInfo $_index all`
                F_add_to_patch_record "$_f_info" "$TMP_SECTION"

            else
                echo "Skip backup ${_fullPath}."
            fi
        #copy to the backup dir
        elif [ -f "$_fullPath" ]; then
            _backup_file_list="$_backup_file_list $_fullPath"
            echo "Add $_fullPath to normal backup."
        else
            echo "Warning: The file $_fullPath is not readable. We won't backup this file."
        fi

        _index=`expr $_index + 1`
    done

    #normal backup
    if [ -n "$_backup_file_list" ] ; then
        echo "Writing backup $BACKUP_TAR."
        tar zcvf "$_backup_dir/$BACKUP_TAR" $_backup_file_list
    else
        echo "No backup files needed. $BACKUP_TAR is not generated."
    fi

    #smart backup
    if [ -n "$_smart_backup_file_list" ] ; then
        echo "Writing smart backup $SMART_BACKUP_TAR."
        tar zcvf "$_backup_dir/$SMART_BACKUP_TAR" -C "${_smart_backup_root}" ${_smart_backup_file_list}
    else
        echo "No smart backup files needed. $SMART_BACKUP_TAR is not generated."
    fi

    #special files backup
    F_special_file_handler "backup" $_backup_dir

    #also save ini and sh
    cp -a *.ini *.sh $_backup_dir

    # Backup patch_management.ini
    cp -a /etc/iscan/patch/config/patch_management.ini $_backup_dir

    return 0
}

#
# F_verify
# Make sure the patch has what it said it contains
#
F_verify(){
    echo "Verifying the files in the patch."

    _index=0
    _install_root=$1
    _number_of_file_items=`F_ini_get $PATCHINFOINI PatchInfo NumberOfFileItems`

    if [ $? -ne 0 ] ; then
        echo "Cannot get number of files in the patch ini."
        return 1
    fi

    while [ "$_index" -lt $_number_of_file_items ]
    do
        _fullPath=`P_get_file_info $PATCHINFOINI PatchInfo $_index path`

        #make sure the patch contains the file it said
        _patchFile=`P_get_file_info $PATCHINFOINI PatchInfo $_index name`
        if [ -z "$_patchFile" ] ; then
            echo "The patch ini file $PATCHINFOINI should have correct file info."
            return 5
        fi

        _patchPath="files/$_patchFile"

        if [ ! -f "$_patchPath" ] ; then
            echo "The patch file $_patchPath was not available. The patch may be corrupted."
            return 4
        fi

        _index=`expr $_index + 1`
    done

    echo "Verification OK."
    return 0
}

#
# F_apply
# If it returns non-zero, make sure it's not a half-way apply. This function should take care of itself.
#
F_apply() {
    echo "Applying the patch files."

    _index=0
    _install_root=$1
    _number_of_file_items=`F_ini_get $PATCHINFOINI PatchInfo NumberOfFileItems`

    if [ $? -ne 0 ] ; then
        echo "Cannot get number of files in the patch ini."
        return 4
    fi

    #change permission to writable
    #chmod -R 644 /usr/iwss/AdminUI/tomcat/webapps/ROOT/html

    #copy the files and set owner, permission
    while [ "$_index" -lt $_number_of_file_items ]
    do
        _fullPath=`P_get_file_info $PATCHINFOINI PatchInfo $_index path`
        _patchFile=`P_get_file_info $PATCHINFOINI PatchInfo $_index name`
        _patchPath="files/$_patchFile"
        _thisVersion=`P_get_file_info $PATCHINFOINI PatchInfo $_index version`

        # don't copy it there is a newer version
        F_detect_previous_patch_file ${_fullPath} ${_thisVersion}

        if [ "$?" = "0" ] ; then
            #smart apply
            _smart_apply=`F_ini_get $PATCHINFOINI PatchInfo SmartApply`
            if [ "$_smart_apply" = "1" ] ; then
                echo "This file ${_fullPath} is kept because it is a newer version."
                _index=`expr $_index + 1`
                continue
            else
                echo "Overwriting newer version file ${_fullPath}."
            fi
        else
            echo "No previous patch file of ${_fullPath}."
        fi

        if [ -L "$_fullPath" ] ; then
            echo "Warning: $_fullPath is a symbolic link."
        fi

        if [ -f "$_fullPath" ] ; then
            printf "%s" "<"
            ls -l $_fullPath
        fi

		#Added for check privilege and owner by anne
		oldPri=$(ls -l $_fullPath|cut -d ' ' -f 1)
		oldUser=$(ls -l $_fullPath|cut -d ' ' -f 3)
		oldGrp=$(ls -l $_fullPath|cut -d ' ' -f 4)

        cp -af $_patchPath $_fullPath
        if [ $? -ne 0 ] ; then
            echo "Unable to overwrite $_fullPath with patch $_patchPath."
            _index=`expr $_index + 1`
            continue
        fi

        #optionally set owner and permission
        _owner=`P_get_file_info $PATCHINFOINI PatchInfo $_index owner`
        if [ -n "$_owner" ] ; then
            chown $_owner $_fullPath
            #if [ "$?" = "0" ] ; then
                #echo "Changed file owner to $_owner."
            #fi
        fi

        _permission=`P_get_file_info $PATCHINFOINI PatchInfo $_index permission`
        if [ -n "$_permission" ] ; then
            chmod $_permission $_fullPath
        fi

        if [ -f "$_fullPath" ] ; then
            printf "%s" ">"
            ls -l $_fullPath
        fi

		#Added for check privilege by anne
		newPri=$(ls -l $_fullPath|cut -d ' ' -f 1)
		newUser=$(ls -l $_fullPath|cut -d ' ' -f 3)
		newGrp=$(ls -l $_fullPath|cut -d ' ' -f 4)
		if [ "$oldPri" == "$newPri" ];then
		   true
		elif [ "" == "$oldPri" ]; then
			echo "warning: Pri not equal--" $_fullPath $oldPri $newPri
		else
			echo "error: Pri not equal--" $_fullPath $oldPri $newPri
		fi
		if [ "$oldUser" == "$newUser" ] ;then
			true
		elif [ "" == "$oldUser" ]; then
			echo "warning: User not equal--" $_fullPath $oldUser $newUser
		else
			echo "error: User not equal--" $_fullPath $oldUser $newUser
		fi

		if [ "$oldGrp" == "$newGrp" ]; then
			true
		elif [ "" == "$oldGrp" ]; then
			echo "warning: Grp not equal--" $_fullPath $oldGrp $newGrp
		else
			echo "error: Grp not equal--" $_fullPath $oldGrp $newGrp
		fi

        # record this file in the patchmanagement.ini
        _f_info=`P_get_file_info $PATCHINFOINI PatchInfo $_index all`
        F_add_to_patch_record "$_f_info" "$TMP_SECTION"

        _index=`expr $_index + 1`
    done
	
	#update server.xml directly
    sed -i 's/protocol="org.apache.coyote.http11.Http11Protocol" maxHttpHeaderSize/protocol="org.apache.coyote.http11.Http11Protocol" maxParameterCount="-1" maxHttpHeaderSize/' /usr/iwss/AdminUI/tomcat/conf/server.xml

    #delete backup if any
    F_ini_delete_section $PATCHMGMINI "${PATCH_NUMBER}_${BACKUP_APPLIED_RECORD_SECTION}"

    #rename current to backup
    F_ini_rename_section $PATCHMGMINI "$APPLIED_RECORD_SECTION" "${PATCH_NUMBER}_${BACKUP_APPLIED_RECORD_SECTION}"

    #rename tmp to current
    F_ini_rename_section $PATCHMGMINI "$TMP_SECTION" "$APPLIED_RECORD_SECTION"

    #special files apply update
    F_special_file_handler apply

    return 0
}

F_update_patch_ini() {
    echo "Updating the patch management ini file."
    _section=`F_ini_get_section $PATCHINFOINI PatchInfo`

    F_ini_append_section $PATCHMGMINI "$PATCH_NUMBER" "$_section"
    if [ $? -ne 0 ] ; then
        return 1
    fi
    F_ini_add_applied_list $PATCHMGMINI "$PATCH_NUMBER"

    #
    # Set the AppliedDate
    #
    _trend_format_date=`date "+%m/%d/%Y %H:%M:%S"`
    F_ini_set $PATCHMGMINI "$PATCH_NUMBER" "AppliedDate"  "$_trend_format_date"

    #
    #
    # Disable all rollbacks except for myself
    # It's means only the last one can be rollback. Even if the last one is rollback, the last second one can't be rollback.
    # This is due to storage constraints.
    #
    #
    _applied_patches=`F_ini_get $PATCHMGMINI PatchManagement AppliedPatches`
    for i in $_applied_patches
    do
        if [ "$PATCH_NUMBER" = "$i" ] ; then
            F_ini_set $PATCHMGMINI "$i" "Rollback" "1"
            #echo "Set $i rollback to 1."
        else
            F_ini_set $PATCHMGMINI "$i" "Rollback" "0"
            #echo "Set $i rollback to 0."
        fi
    done
}

F_check_required_fields()
{
    if [ ! -f "$PATCHMGMINI" ] ; then
        echo "Cannot find $PATCHMGMINI."
        return 1
    fi

    if [ ! -f "$PATCHINFOINI" ] ; then
        echo "Cannot find $PATCHINFOINI."
        return 1
    fi

    _number_of_file_items=`F_ini_get $PATCHINFOINI PatchInfo NumberOfFileItems`
    if [ $? -ne 0 ] ; then
        echo "Cannot find [PatchInfo] Number of FileOfFileItems in $PATCHINFOINI."
        return 1
    fi

    return 0
}

F_check_build_number()
{
    _patch_build_number=`F_ini_get $PATCHINFOINI PatchInfo Build`
    _current_build_number=`/usr/iwss/iwssd -v | cut -d" " -f2 | cut -d_ -f4`
    #_current_sp_number=`/usr/iwss/iwssd -v | grep SP1 |wc -l`

    if [ "" == "$_current_build_number" ] ; then
        echo "Failed to retrieve installed IWSS build number"
        return 1
    fi

    if [ "" == "$_patch_build_number" ] ; then
        echo "Cannot get build number for this patch."
        return 1
    fi
    #if [ 0 -eq $(($_current_sp_number)) ] ; then
    #    echo "SP1 is not installed, please install SP1 then try again."
    #    return 2
    #fi
    if [ $(($_current_build_number)) -ge $(($_patch_build_number)) ] ; then
        echo "Installed build number ($_current_build_number) is greater than or equal to the patch build number ($_patch_build_number)"
        return 3
    fi

    echo "Build number check OK ($_patch_build_number for this patch vs $_current_build_number installed)"

    return 0
}

F_check_build_minimum()
{
    #checking for MinOsPatchNum
    _current_os_build_number=`cat /etc/ver.os | cut -d"." -f3`
    _patch_required_os_number=`F_ini_get $PATCHINFOINI PatchInfo MinOsPatchNum`

    if [ 0 -eq $(($_patch_required_os_number)) ] ; then
        echo "Cannot get required MinOsPatchNum for this patch.  Install anyway."
        return 0
    fi

    if [ $(($_current_os_build_number)) -lt $(($_patch_required_os_number)) ] ; then
        echo "Current OS build ($_current_build_number) is older than the required build ($_patch_required_os_number)"
        echo "Please patch your OS to at least build ($_patch_required_os_number) before applying this update"
        return 2
    fi

    #herb_todo add MinAppPatchNum checking
    _current_app_build_number=`/usr/iwss/iwssd -v | cut -d" " -f2 | cut -d_ -f4`

    _patch_required_app_number=`F_ini_get $PATCHINFOINI PatchInfo MinBld`
    if [ 0 -eq $(($_patch_required_app_number)) ] ; then
        echo "Cannot get required MinAppPatchNum for this patch.  Install anyway."
        return 0
    fi

    if [ $(($_current_app_build_number)) -lt $(($_patch_required_app_number)) ] ; then
        echo "Current App build ($_current_app_build_number) is older than the required build ($_patch_required_app_number)"
        echo "Please patch your Application to at least build ($_patch_required_app_number) before applying this update"
        return 2
    fi

    return 0
}

F_check_dependency() {
    #echo "Checking Dependency."

    _applied_patches=`F_ini_get $PATCHMGMINI PatchManagement AppliedPatches`

    #stop if the patch number is already in AppliedPatches..
    for i in $_applied_patches
    do
        if [ "$PATCH_NUMBER" = "$i" ] ; then
            echo "Cannot apply this patch. The patch script has detected a same patch number."
            return 2
        fi
    done

    #delete the section if there is an old section
    F_ini_get_section $PATCHMGMINI $PATCH_NUMBER > /dev/null 2>&1
    if [ $? -eq 0 ] ; then
        echo "The patch section is still in $PATCHMGMINI. The old section will overwritten."

        F_ini_delete_section $PATCHMGMINI $PATCH_NUMBER
    fi

    echo "Dependency Check OK."
    return 0
}

F_check_version()
{
    AP_version=`/usr/iwss/iwssd -v | cut -d ' ' -f2 | cut -d '_' -f1`

    echo "AP_version = $AP_version"

    if [ "$AP_version" != "6.5-SP2" ] ; then
        echo "IWSS version is wrong. This patch is for version 6.5-SP2"
        return 1
    fi

    echo "Prodcut version check OK (version 6.5-SP2 installed)"
    return 0
}

F_check_requirement() {
    #Return error if I require a patch that doesn't exist.

    _requires_number=`F_ini_get $PATCHINFOINI PatchInfo Requires`

    if [ -z "$_requires_number" ] ; then
        return 0
    fi

    if [ -n "$_requires_number" ] ; then
        _found_require=0

        for j in $_applied_patches
        do
            if [ "$_requires_number" = "$j" ] ; then
                _found_require=1
            fi
        done

        if [ "$_found_require" = "0" ] ; then
            echo "Cannot find the required previous patch $_requires_number."
            return 3
        fi
    fi

    return 0
}

F_upload_check() {

    # very important: check this call is coming from APP patch
    if [ $APP_PATCH_ROOT != $INSTALL_DIR ]; then
        F_log "The patch is not used for APP patch. APP_PATCH_ROOT=$APP_PATCH_ROOT, INSTALL_DIR=$INSTALL_DIR"
        exit $ERR_PATCH_APP
    fi

    F_check_required_fields
    if [ $? -ne 0 ] ; then
        F_log "The patch format is incorrect. Please check required fields in the configuration file."
        exit $ERR_REQ
    fi

    F_check_build_number
    _result=$?
    if [ $_result -ne 0 ] ; then
        F_log "Cannot pass build check. Exit. ?=$_result"
        if [ $_result == 1 ];then
            exit $ERR_REQ_F
        elif [ $_result == 2 ];then
            exit $ERR_REQ_P
        else
            exit $ERR_DEP
        fi
    fi

    F_check_dependency
    if [ $? -ne 0 ] ; then
        F_log "Cannot pass dependency check. Exit."
        exit $ERR_DEP
    fi

    F_check_version
    if [ $? -ne 0 ] ; then
        F_log "Cannot pass version check. Exit."
        exit $ERR_REQ_P
    fi

    F_check_build_minimum
    if [ $? -ne 0 ] ; then
        F_log "Cannot pass minimum build check. Exit."
        exit $ERR_REQ_P
    fi

    F_check_requirement
    if [ $? -ne 0 ] ; then
        F_log "Cannot pass requirement check. Exit."
        exit $ERR_REQ_P
    fi

    F_verify        $INSTALL_DIR
    if [ $? -ne 0 ] ; then
        F_log "Cannot verify the patch. Exit."
        exit $ERR_VER
    fi
}

F_apply_now() {

    F_backup        $INSTALL_DIR $BACKUP_DIR
    if [ $? -ne 0 ] ; then
        F_log "Cannot backup the system successfully. Exit."
        exit $ERR_BAK
    fi

    _current_build_number=`/usr/iwss/iwssd -v | cut -d" " -f2 | cut -d_ -f4`

	F_apply_database_changes

    F_stop_daemons  $INSTALL_DIR

    echo "Clear the tomcat cache."
    rm -rf /usr/iwss/AdminUI/tomcat/work/Standalone/localhost/_/*

    F_apply $INSTALL_DIR $BACKUP_DIR
    if [ $? -ne 0  ] ; then
        F_log "Cannot apply patch successfully. Rolling back."
        F_rollback $INSTALL_DIR $BACKUP_DIR

        echo "Start the daemons and exit."
        F_start_daemons $INSTALL_DIR
        exit $ERR_APP
    fi

	F_apply_extra_changes

    F_update_patch_ini $INSTALL_DIR

    #if possible, rollback ini when failed....
    F_start_daemons $INSTALL_DIR

    F_log "The patch script is finished."

    exit 0
}

if [ "$CHECK_OR_APPLY" = "apply_now" ] ; then
    #check again just to make sure
    F_upload_check
    F_apply_now
elif [ "$CHECK_OR_APPLY" = "upload_check" ] ; then
    F_upload_check
fi
