#!/bin/sh

#herb_todo
echo "p0=$0, p1=$1, p2=$2, p3=$3" >> /tmp/ttt

#Warning on Stargate behaviors:
# 1. Web UI is a compressed file and it was decompressed at boot. So, any direct overwrite won't work in the next boot.
# 2. Factory default config is compressed, too.

F_usage() {
	echo "Usage: $0 install_root_dir backup_dir"
	echo "This script should stop the daemons, rollback for this patch, and start the daemon."
}

INSTALL_DIR="$1"
if [ -z "$INSTALL_DIR" ] ; then
	echo "1nd argument should be installation directory."
	F_usage
	exit 2
fi

BACKUP_DIR="$2"
if [ -z "$BACKUP_DIR" ] ; then
	echo "2nd argument should be backup directory."
	F_usage
	exit 3
fi

. ./ini_util.sh
. ./stargate_patch_common.sh
. ./patch_action_extra.sh

PATCHINFOINI=stargate_patch.ini
PATCHMGMINI=${BACKUP_DIR}/../../config/patch_management.ini

#F_rollback is in another file because apply needs it too.

F_update_ini_rollback() {
	echo "Updating the patch management ini file."

	_section=`F_ini_get_section $PATCHINFOINI PatchInfo`
	_patch_number=`F_ini_get $PATCHINFOINI PatchInfo Number`

	if [ $? -ne 0 ] ; then
		echo "Cannot get patch number from ${PATCHINFOINI}"
		return 2
	fi

	F_ini_del_from_applied_list $PATCHMGMINI "$_patch_number"
	F_ini_delete_section $PATCHMGMINI "$_patch_number"

	F_ini_exist_section ${PATCHMGMINI} "${PATCH_NUMBER}_${BACKUP_APPLIED_RECORD_SECTION}"
	if [ "$?" = "0" ] ; then
		echo "Recover patch record from section ${PATCH_NUMBER}_${BACKUP_APPLIED_RECORD_SECTION}."
		F_ini_rename_section ${PATCHMGMINI} "$APPLIED_RECORD_SECTION" "TO_BE_DELETE"
		F_ini_rename_section ${PATCHMGMINI} "${PATCH_NUMBER}_${BACKUP_APPLIED_RECORD_SECTION}" "$APPLIED_RECORD_SECTION"
		F_ini_delete_section ${PATCHMGMINI} "TO_BE_DELETE"
	else
		echo "No need to recover ${PATCH_NUMBER}_${BACKUP_APPLIED_RECORD_SECTION}."
	fi

	# Restore patch_management.ini
	if [ -f $BACKUP_DIR/patch_management.ini ]; then
		cp -f $BACKUP_DIR/patch_management.ini /etc/iscan/patch/config
	fi

	return 0
}

F_check_dependency_rollback() {

	#Stops if some body else requires me.

	_applied_patches=`F_ini_get $PATCHMGMINI PatchManagement AppliedPatches`
	_patch_number=`F_ini_get $PATCHINFOINI PatchInfo Number`

	#stop if the patch number is already in AppliedPatches..
	for i in $_applied_patches
	do
		if [ "$i" = "$_patch_number" ] ; then
			continue
		fi
		_this_requires=`F_ini_get $PATCHMGMINI $i Requires`
		if [ -n "$_this_requires" ] ; then
			for j in $_this_requires
			do
				if [ "$j" = "$_patch_number" ] ; then
					echo "Cannot rollback. Patch $j requires this patch $_patch_number."
					return 3
				fi
			done
		fi
	done

	echo "Dependency Check OK."
	return 0
}

F_ini_list_section()
{
    INI_F="$1"

    if [ ! -f "$INI_F" ] ; then
        echo "Error: ${INI_F} not found."
        return 1
    fi

    _R=`cat $1 | grep  '\[.*\]' | cut -d '[' -f 2 | cut -d ']' -f 1`

}

F_check_dependency_rollback
if [ $? -ne 0 ] ; then
	echo "Cannot pass rollback dependency check. Exit ..."
	exit 2
fi

F_rollback_database_changes

F_stop_daemons 	$INSTALL_DIR

#################################################
# Custom action for IWSS patch

# Remove all pre-compiled Tomcat class files
rm -rf /usr/iwss/AdminUI/tomcat/work/Standalone/localhost/_/*
#################################################

rm -f /usr/iwss/AdminUI/tomcat/webapps/ROOT/WEB-INF/lib/IWSSPatch.jar
rm -f /usr/iwss/AdminUI/tomcat/webapps/ROOT/systemRebootMsg_migration.jsp
rm -f /usr/iwss/AdminUI/tomcat/webapps/ROOT/systemRebootMsg_rollback.jsp

F_rollback	$INSTALL_DIR $BACKUP_DIR
if [ $? -ne 0 ] ; then
	echo "Cannot rollback the system successfully. Exit ..."
	exit 3
fi

F_update_ini_rollback    $INSTALL_DIR

F_rollback_extra_changes

# Remove factory conf
if [ -f /etc/iscan/patch/config/patch_management.ini.factory ] ; then
	rm -f /etc/iscan/patch/config/patch_management.ini.factory
fi

F_start_daemons $INSTALL_DIR

#delete backups
rm -rf $BACKUP_DIR

